# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from .teach_base_page import TeachBasePage


class TeachClassPage(TeachBasePage):
    def __init__(self, browser):
        super(TeachClassPage, self).__init__(browser)

        assert self.on_correct_page("teach_class_page")

    def type_student_name(self, name):
        self.browser.find_element_by_id("id_names").send_keys(name + "\n")
        return self

    def create_students(self):
        self._click_create_students()

        import portal.tests.pageObjects.portal.teach.onboarding_student_list_page as onboarding_student_list_page

        return onboarding_student_list_page.OnboardingStudentListPage(self.browser)

    def _click_create_students(self):
        self.browser.find_element_by_name("new_students").click()

    def student_exists(self, name):
        return name in self.browser.find_element_by_id("student_table").text

    def delete_class(self):
        self.browser.find_element_by_id("deleteClass").click()
        return self

    def delete_students(self):
        self.browser.find_element_by_id("deleteSelectedStudents").click()
        return self

    def reset_passwords(self):
        self.browser.find_element_by_id("resetSelectedStudents").click()
        return self

    def move_students(self):
        self.browser.find_element_by_id("moveSelectedStudents").click()

        import portal.tests.pageObjects.portal.teach.move_students_page as move_students_page

        return move_students_page.TeachMoveStudentsPage(self.browser)

    def move_students_none_selected(self):
        self.browser.find_element_by_id("moveSelectedStudents").click()

        return self

    def dismiss_students(self):
        self.browser.find_element_by_id("dismissSelectedStudents").click()

        import portal.tests.pageObjects.portal.teach.dismiss_students_page as dismiss_students_page

        return dismiss_students_page.TeachDismissStudentsPage(self.browser)

    def confirm_delete_class_dialog(self):
        self.confirm_dialog()

        import portal.tests.pageObjects.portal.teach.dashboard_page as dashboard_page

        return dashboard_page.TeachDashboardPage(self.browser)

    def confirm_delete_student_dialog(self):
        self.confirm_dialog()

        return self

    def confirm_reset_student_dialog(self):
        self.confirm_dialog()

        import portal.tests.pageObjects.portal.teach.onboarding_student_list_page as onboarding_student_list_page

        return onboarding_student_list_page.OnboardingStudentListPage(self.browser)

    def confirm_dialog_expect_error(self):
        self.confirm_dialog()

        return self

    def toggle_select_student(self):
        self.browser.find_element_by_id("student_checkbox").click()
        return self

    def wait_for_messages(self):
        self.wait_for_element_by_id("messages")

    def has_students(self):
        return self.element_exists_by_id("student_table")

    def go_to_class_settings_page(self):
        self.browser.find_element_by_id("class_settings_button").click()

        import portal.tests.pageObjects.portal.teach.class_settings_page as class_settings_page

        return class_settings_page.TeachClassSettingsPage(self.browser)

    def go_to_edit_student_page(self):
        self.browser.find_element_by_id("edit_student_button").click()

        import portal.tests.pageObjects.portal.teach.edit_student_page as edit_student_page

        return edit_student_page.EditStudentPage(self.browser)

    def go_to_dashboard(self):
        self.browser.find_element_by_id("return_to_classes_button").click()

        import portal.tests.pageObjects.portal.teach.dashboard_page as dashboard_page

        return dashboard_page.TeachDashboardPage(self.browser)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import re
from datetime import timedelta

from captcha.fields import ReCaptchaField
from captcha.widgets import ReCaptchaV2Invisible
from common.models import Class, Student, stripStudentName
from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm
from django.utils import timezone

from portal.helpers.password import form_clean_password
from common.permissions import logged_in_as_independent_student


class StudentLoginForm(AuthenticationForm):
    username = forms.CharField(
        label="Name", widget=forms.TextInput(attrs={"placeholder": "Jane"})
    )
    access_code = forms.CharField(
        label="Class Access Code",
        widget=forms.TextInput(attrs={"placeholder": "AB123"}),
    )
    password = forms.CharField(label="Password", widget=forms.PasswordInput)

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    error_messages = {
        "invalid_login": "Invalid name, class access code or password",
        "inactive": "This account is inactive.",
    }

    def clean(self):
        name = self.cleaned_data.get("username", None)
        access_code = self.cleaned_data.get("access_code", None)
        password = self.cleaned_data.get("password", None)

        if name and access_code and password:

            student, user = self.check_for_errors(name, access_code, password)

            self.student = student
            self.user_cache = user
        return self.cleaned_data

    def check_for_errors(self, name, access_code, password):
        classes = Class.objects.filter(access_code__iexact=access_code)
        if len(classes) != 1:
            raise forms.ValidationError("Invalid name, class access code or password")

        name = stripStudentName(name)

        students = Student.objects.filter(
            new_user__first_name__iexact=name, class_field=classes[0]
        )
        if len(students) != 1:
            raise forms.ValidationError("Invalid name, class access code or password")

        student = students[0]
        user = authenticate(username=student.new_user.username, password=password)

        if user is None:
            raise forms.ValidationError("Invalid name, class access code or password")
        if not user.is_active:
            raise forms.ValidationError("This user account has been deactivated")

        return student, user


class StudentEditAccountForm(forms.Form):
    password = forms.CharField(
        label="New password", required=True, widget=forms.PasswordInput
    )
    confirm_password = forms.CharField(
        label="Confirm new password", required=True, widget=forms.PasswordInput
    )
    current_password = forms.CharField(
        label="Current password", widget=forms.PasswordInput
    )

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(StudentEditAccountForm, self).__init__(*args, **kwargs)

    def clean_password(self):
        return form_clean_password(self, forms, "password")

    def clean(self):
        return clean_confirm_password(self)


class IndependentStudentEditAccountForm(forms.Form):
    name = forms.CharField(
        label="Name",
        max_length=100,
        required=False,
        widget=forms.TextInput(attrs={"placeholder": "Name"}),
    )
    email = forms.EmailField(
        label="New email address (optional)",
        required=False,
        widget=forms.EmailInput(attrs={"placeholder": "new.address@myemail.com"}),
    )
    password = forms.CharField(
        label="New password", required=False, widget=forms.PasswordInput
    )
    confirm_password = forms.CharField(
        label="Confirm new password", required=False, widget=forms.PasswordInput
    )
    current_password = forms.CharField(
        label="Current password", widget=forms.PasswordInput
    )

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(IndependentStudentEditAccountForm, self).__init__(*args, **kwargs)

    def clean_name(self):
        name = self.cleaned_data.get("name", None)
        if not self.user.new_student.class_field:
            if name == "":
                raise forms.ValidationError("This field is required")

            if re.match(re.compile("^[\w ]+$"), name) is None:
                raise forms.ValidationError(
                    "Names may only contain letters, numbers, dashes, underscores, and spaces."
                )

        return name

    def clean_password(self):
        return form_clean_password(self, forms, "password")

    def clean(self):
        return clean_confirm_password(self)


def clean_confirm_password(self):
    password = self.cleaned_data.get("password", None)
    confirm_password = self.cleaned_data.get("confirm_password", None)
    current_password = self.cleaned_data.get("current_password", None)

    if are_password_and_confirm_password_different(password, confirm_password):
        raise forms.ValidationError("Your new passwords do not match")

    if current_password and not self.user.check_password(current_password):
        raise forms.ValidationError("Your current password was incorrect")

    return self.cleaned_data


def are_password_and_confirm_password_different(password, confirm_password):
    return password is not None and password != confirm_password


class IndependentStudentSignupForm(forms.Form):
    name = forms.CharField(
        label="Name",
        max_length=100,
        widget=forms.TextInput(attrs={"placeholder": "Rosalind Franklin"}),
    )

    username = forms.CharField(
        label="Username",
        max_length=100,
        widget=forms.TextInput(attrs={"placeholder": "rosie_f"}),
    )

    email = forms.EmailField(
        label="Email address",
        widget=forms.EmailInput(
            attrs={"placeholder": "rosalind.franklin@cambridge.ac.uk"}
        ),
    )

    newsletter_ticked = forms.BooleanField(initial=False, required=False)

    password = forms.CharField(label="Password", widget=forms.PasswordInput)

    confirm_password = forms.CharField(
        label="Confirm Password", widget=forms.PasswordInput
    )

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    def clean_name(self):
        name = self.cleaned_data.get("name", None)
        if re.match(re.compile("^[\w ]+$"), name) is None:
            raise forms.ValidationError(
                "Names may only contain letters, numbers, dashes, underscores, and spaces."
            )

        return name

    def clean_username(self):
        username = self.cleaned_data.get("username", None)

        if re.match(re.compile("[\w]+"), username) is None:
            raise forms.ValidationError(
                "Usernames may only contain letters, numbers, dashes, and underscores."
            )

        return username

    def clean_password(self):
        return form_clean_password(self, forms, "password")

    def clean(self):
        password = self.cleaned_data.get("password", None)
        confirm_password = self.cleaned_data.get("confirm_password", None)

        if password and confirm_password and password != confirm_password:
            raise forms.ValidationError("Your passwords do not match")

        return self.cleaned_data


class IndependentStudentLoginForm(AuthenticationForm):
    username = forms.CharField(
        label="Username", widget=forms.TextInput(attrs={"placeholder": "rosie_f"})
    )
    password = forms.CharField(label="Password", widget=forms.PasswordInput())

    captcha = ReCaptchaField(widget=ReCaptchaV2Invisible)

    def confirm_login_allowed(self, user):
        if not logged_in_as_independent_student(user):
            raise forms.ValidationError("Incorrect username or password")


class StudentJoinOrganisationForm(forms.Form):
    access_code = forms.CharField(
        label="Class Access Code",
        widget=forms.TextInput(attrs={"placeholder": "AB123"}),
    )

    def clean(self):
        access_code = self.cleaned_data.get("access_code", None)

        if access_code:
            classes = Class.objects.filter(access_code=access_code)
            if len(classes) != 1:
                raise forms.ValidationError(
                    "Cannot find the school or club and/or class"
                )
            self.klass = classes[0]
            if not self.klass.always_accept_requests:
                if self.klass.accept_requests_until is None:
                    raise forms.ValidationError(
                        "Cannot find the school or club and/or class"
                    )
                elif (self.klass.accept_requests_until - timezone.now()) < timedelta():
                    raise forms.ValidationError(
                        "Cannot find the school or club and/or class"
                    )
        return self.cleaned_data



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

import time

from aimmo.models import Game, Worksheet
from django.core import mail
from django.core.urlresolvers import reverse
from django.test import Client, TestCase
from selenium.webdriver.support.wait import WebDriverWait

from common.models import Class, Student, Teacher
from common.tests.utils import email as email_utils
from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import (
    create_independent_student_directly,
    create_school_student_directly,
)
from common.tests.utils.teacher import (
    signup_duplicate_teacher_fail,
    signup_teacher,
    signup_teacher_directly,
    submit_teacher_signup_form,
)

from .base_test import BaseTest
from .pageObjects.portal.home_page import HomePage
from .utils.messages import (
    is_email_verified_message_showing,
    is_teacher_details_updated_message_showing,
    is_teacher_email_updated_message_showing,
)
from .utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)


class TestTeachers(TestCase):
    def test_new_student_can_play_games(self):
        """
        Given a teacher has an Kurono game,
        When they add a new student to their class,
        Then the new student should be able to play that class's games
        """
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        klass, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        c = Client()
        c.login(username=email, password=password)
        worksheet: Worksheet = Worksheet.objects.create(
            name="test", starter_code="test"
        )
        c.post(
            reverse("kurono"),
            {"name": "Test Game", "game_class": klass.pk, "worksheet": worksheet.id},
        )
        c.post(
            reverse("view_class", kwargs={"access_code": access_code}),
            {"names": "Florian"},
        )

        game = Game.objects.get(id=1)
        new_student = Student.objects.last()
        assert game.can_user_play(new_student.new_user)

    def test_accepted_independent_student_can_play_games(self):
        """
        Given an independent student requests access to a class,
        When the teacher for that class accepts the request,
        Then the new student should have access to that class's games
        """
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        klass, _, access_code = create_class_directly(email)
        klass.always_accept_requests = True
        klass.save()
        create_school_student_directly(access_code)
        (
            indep_username,
            indep_password,
            indep_student,
        ) = create_independent_student_directly()

        c = Client()

        c.login(username=indep_username, password=indep_password)
        c.post(
            reverse("student_join_organisation"),
            {"access_code": access_code, "class_join_request": "Request"},
        )
        c.logout()

        c.login(username=email, password=password)
        worksheet: Worksheet = Worksheet.objects.create(
            name="test", starter_code="test"
        )
        c.post(
            reverse("kurono"),
            {"name": "Test Game", "game_class": klass.pk, "worksheet": worksheet.id},
        )
        c.post(
            reverse("teacher_accept_student_request", kwargs={"pk": indep_student.pk}),
            {"name": "Florian"},
        )

        game: Game = Game.objects.get(id=1)
        new_student = Student.objects.last()
        assert game.can_user_play(new_student.new_user)

    def test_moved_class_has_correct_permissions_for_students_and_teachers(self):
        """
        Given two teachers each with a class and an aimmo game,
        When teacher 1 transfers their class to teacher 2,
        Then:
            - Students in each class still only have access to their class games
            - Teacher 2 has access to both games and teacher 1 has access to none
        """
        worksheet: Worksheet = Worksheet.objects.create(
            name="test", starter_code="test"
        )

        # Create teacher 1 -> class 1 -> student 1
        email1, password1 = signup_teacher_directly()
        school_name, postcode = create_organisation_directly(email1)
        klass1, _, access_code1 = create_class_directly(email1, True)
        create_school_student_directly(access_code1)

        # Create teacher 2 -> class 2 -> student 2
        email2, password2 = signup_teacher_directly()
        join_teacher_to_organisation(email2, school_name, postcode)
        klass2, _, access_code2 = create_class_directly(email2, True)
        create_school_student_directly(access_code2)

        teacher1: Teacher = Teacher.objects.get(new_user__email=email1)
        teacher2: Teacher = Teacher.objects.get(new_user__email=email2)

        c = Client()

        # Create game 1 under class 1
        c.login(username=email1, password=password1)
        c.post(
            reverse("kurono"),
            {"name": "Game 1", "game_class": klass1.pk, "worksheet": worksheet.id},
        )
        c.logout()

        # Create game 2 under class 2
        c.login(username=email2, password=password2)
        c.post(
            reverse("kurono"),
            {"name": "Game 2", "game_class": klass2.pk, "worksheet": worksheet.id},
        )
        c.logout()

        game1: Game = Game.objects.get(owner=teacher1.new_user)
        game2: Game = Game.objects.get(owner=teacher2.new_user)

        student1: Student = Student.objects.get(class_field=klass1)
        student2: Student = Student.objects.get(class_field=klass2)

        # Check student permissions for each game
        assert game1.can_user_play(student1.new_user)
        assert game2.can_user_play(student2.new_user)
        assert not game1.can_user_play(student2.new_user)
        assert not game2.can_user_play(student1.new_user)

        # Check teacher permissions for each game
        assert game1.can_user_play(teacher1.new_user)
        assert game2.can_user_play(teacher2.new_user)
        assert not game1.can_user_play(teacher2.new_user)
        assert not game2.can_user_play(teacher1.new_user)

        # Transfer class 1 over to teacher 2
        c.login(username=email1, password=password1)
        response = c.post(
            reverse("teacher_move_class", kwargs={"access_code": access_code1}),
            {"new_teacher": teacher2.pk},
        )
        assert response.status_code == 302
        c.logout()

        # Refresh model instances
        klass1: Class = Class.objects.get(pk=klass1.pk)
        klass2: Class = Class.objects.get(pk=klass2.pk)
        game1 = Game.objects.get(pk=game1.pk)
        game2 = Game.objects.get(pk=game2.pk)

        # Check teacher 2 is the teacher for class 1
        assert klass1.teacher == teacher2

        # Check that the students' permissions have not changed
        assert game1.can_user_play(student1.new_user)
        assert game2.can_user_play(student2.new_user)
        assert not game1.can_user_play(student2.new_user)
        assert not game2.can_user_play(student1.new_user)

        # Check that teacher 1 cannot access class 1's game 1 anymore
        assert not game1.can_user_play(teacher1.new_user)

        # Check that teacher 2 can access game 1
        assert game1.can_user_play(teacher2.new_user)

    def test_moved_student_has_access_to_only_new_teacher_games(self):
        """
        Given a student in a class,
        When a teacher transfers them to another class with a new teacher,
        Then the student should only have access to the new teacher's games
        """
        worksheet: Worksheet = Worksheet.objects.create(
            name="test", starter_code="test"
        )

        email1, password1 = signup_teacher_directly()
        school_name, postcode = create_organisation_directly(email1)
        klass1, _, access_code1 = create_class_directly(email1, True)
        create_school_student_directly(access_code1)

        email2, password2 = signup_teacher_directly()
        join_teacher_to_organisation(email2, school_name, postcode)
        klass2, _, access_code2 = create_class_directly(email2, True)
        create_school_student_directly(access_code2)

        teacher1 = Teacher.objects.get(new_user__email=email1)
        teacher2 = Teacher.objects.get(new_user__email=email2)

        c = Client()
        c.login(username=email2, password=password2)
        c.post(
            reverse("kurono"),
            {"name": "Game 2", "game_class": klass2.pk, "worksheet": worksheet.id},
        )
        c.logout()

        c.login(username=email1, password=password1)
        c.post(
            reverse("kurono"),
            {"name": "Game 1", "game_class": klass1.pk, "worksheet": worksheet.id},
        )

        game1 = Game.objects.get(owner=teacher1.new_user)
        game2 = Game.objects.get(owner=teacher2.new_user)

        student1 = Student.objects.get(class_field=klass1)
        student2 = Student.objects.get(class_field=klass2)

        self.assertTrue(game1.can_user_play(student1.new_user))
        self.assertTrue(game2.can_user_play(student2.new_user))

        c.post(
            reverse("teacher_move_students", kwargs={"access_code": access_code1}),
            {"transfer_students": student1.pk},
        )
        c.post(
            reverse(
                "teacher_move_students_to_class", kwargs={"access_code": access_code1}
            ),
            {
                "form-0-name": student1.user.user.first_name,
                "form-MAX_NUM_FORMS": 1000,
                "form-0-orig_name": student1.user.user.first_name,
                "form-TOTAL_FORMS": 1,
                "form-MIN_NUM_FORMS": 0,
                "submit_disambiguation": "",
                "form-INITIAL_FORMS": 1,
                "new_class": klass2.pk,
            },
        )
        c.logout()

        game1 = Game.objects.get(owner=teacher1.new_user)
        game2 = Game.objects.get(owner=teacher2.new_user)

        self.assertTrue(not game1.can_user_play(student1.new_user))
        self.assertTrue(game2.can_user_play(student1.new_user))


class TestTeacher(BaseTest):
    def test_signup_without_newsletter(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, _, _ = signup_teacher(page)
        assert is_email_verified_message_showing(self.selenium)

    def test_signup_with_newsletter(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, _, _ = signup_teacher(page, newsletter=True)
        assert is_email_verified_message_showing(self.selenium)

    def test_signup_duplicate_failure(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, email, _ = signup_teacher(page)
        assert is_email_verified_message_showing(self.selenium)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, _, _ = signup_duplicate_teacher_fail(page, email)
        assert page.__class__.__name__ == "TeacherLoginPage"

    def test_signup_failure_short_password(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = submit_teacher_signup_form(page, password="test")
        assert page.has_teacher_signup_failed(
            "Password not strong enough, consider using at least 8 characters, upper and lower case letters, and numbers"
        )

    def test_signup_failure_common_password(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = submit_teacher_signup_form(page, password="Password1")
        assert page.has_teacher_signup_failed(
            "Password not strong enough, consider using at least 8 characters, upper and lower case letters, and numbers"
        )

    def test_login_failure(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_teacher_login_page()
        page = page.login_failure(
            "non-existent-email@codeforlife.com", "Incorrect password"
        )
        assert page.has_login_failed(
            "form-login-teacher", "Incorrect email address or password"
        )

    def test_login_success(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_teacher_login_page()
        page = page.login(email, password)
        assert self.is_dashboard_page(page)

    def test_signup_login_success(self):
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page, email, password = signup_teacher(page)
        page = page.login_no_school(email, password)
        assert self.is_onboarding_page(page)

    def test_view_resources(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)
        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium)
        page = page.go_to_teacher_login_page()
        page = page.login(email, password)

        assert self.is_dashboard_page(page)

        page = page.go_to_resources_page().go_to_materials_page()

        assert self.is_materials_page(page)

        page = page.click_pdf_link()

        assert self.is_pdf_viewer_page(page)

        page = page.click_resources_button_link().go_to_materials_page()

        assert self.is_materials_page(page)

    def test_edit_details(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)

        page = page.change_teacher_details(
            {
                "title": "Mrs",
                "first_name": "Paulina",
                "last_name": "Koch",
                "current_password": "Password2",
            }
        )
        assert self.is_dashboard_page(page)
        assert is_teacher_details_updated_message_showing(self.selenium)

        assert page.check_account_details(
            {"title": "Mrs", "first_name": "Paulina", "last_name": "Koch"}
        )

    def test_edit_details_non_admin(self):
        email_1, _ = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)
        _, _, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)
        join_teacher_to_organisation(email_2, name, postcode)
        _, _, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email_2, password_2)
        )

        page = page.change_teacher_details(
            {
                "title": "Mr",
                "first_name": "Florian",
                "last_name": "Aucomte",
                "current_password": "Password2",
            }
        )
        assert self.is_dashboard_page(page)
        assert is_teacher_details_updated_message_showing(self.selenium)

        assert page.check_account_details(
            {"title": "Mr", "first_name": "Florian", "last_name": "Aucomte"}
        )

    def test_change_email(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)

        new_email = "another-email@codeforlife.com"
        page = page.change_email("Test", "Teacher", new_email, password)
        assert page.__class__.__name__ == "EmailVerificationNeededPage"
        assert is_teacher_email_updated_message_showing(self.selenium)

        page = email_utils.follow_change_email_link_to_dashboard(page, mail.outbox[0])
        mail.outbox = []

        page = page.login(new_email, password)

        assert page.check_account_details(
            {"title": "Mr", "first_name": "Test", "last_name": "Teacher"}
        )

    def test_reset_password(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.get_to_forgotten_password_page()

        page.reset_email_submit(email)

        self.wait_for_email()

        page = email_utils.follow_reset_email_link(self.selenium, mail.outbox[0])

        new_password = "AnotherPassword12"

        page.teacher_reset_password(new_password)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email, new_password)
        )
        assert self.is_dashboard_page(page)

    def test_reset_password_fail(self):
        page = self.get_to_forgotten_password_page()
        fake_email = "fake_email@fakeemail.com"
        page.reset_email_submit(fake_email)

        time.sleep(5)

        assert len(mail.outbox) == 0

    def get_to_forgotten_password_page(self):
        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .go_to_teacher_forgotten_password_page()
        )
        return page

    def wait_for_email(self):
        WebDriverWait(self.selenium, 2).until(lambda driver: len(mail.outbox) == 1)

    def is_dashboard_page(self, page):
        return page.__class__.__name__ == "TeachDashboardPage"

    def is_materials_page(self, page):
        return page.__class__.__name__ == "MaterialsPage"

    def is_pdf_viewer_page(self, page):
        return page.__class__.__name__ == "PDFViewerPage"

    def is_onboarding_page(self, page):
        return page.__class__.__name__ == "OnboardingOrganisationPage"



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.


def is_message_showing(browser, message):
    return message in browser.find_element_by_id("messages").text


def is_email_verified_message_showing(browser):
    return is_message_showing(
        browser, "Your email address was successfully verified, please log in."
    )


def is_teacher_details_updated_message_showing(browser):
    return is_message_showing(
        browser, "Your account details have been successfully changed."
    )


def is_student_details_updated_message_showing(browser):
    return is_message_showing(
        browser, "Your account details have been changed successfully."
    )


def is_indep_student_join_request_received_message_showing(browser):
    return is_message_showing(
        browser, "Your request to join a school has been received successfully."
    )


def is_indep_student_join_request_revoked_message_showing(browser):
    return is_message_showing(
        browser, "Your request to join a school has been cancelled successfully."
    )


def is_teacher_email_updated_message_showing(browser):
    return is_message_showing(
        browser,
        "Your account details have been successfully changed. Your email will be "
        "changed once you have verified it, until then you can still log in with your "
        "old email.",
    )


def is_organisation_created_message_showing(browser, name):
    return is_message_showing(
        browser, "The school or club '%s' has been successfully added." % name
    )


def is_class_created_message_showing(browser, name):
    return is_message_showing(
        browser, "The class '%s' has been created successfully." % name
    )


def is_class_nonempty_message_showing(browser):
    return is_message_showing(
        browser,
        "This class still has students, please remove or delete them all before "
        "deleting the class.",
    )


def is_contact_message_sent_message_showing(browser):
    return is_message_showing(browser, "Your message was sent successfully.")



from django.contrib.auth.views import LoginView
from portal.forms.play import IndependentStudentLoginForm
from django.core.urlresolvers import reverse_lazy


class IndependentStudentLoginView(LoginView):
    template_name = "portal/login/independent_student.html"
    form_class = IndependentStudentLoginForm
    success_url = reverse_lazy("student_details")
    redirect_authenticated_user = reverse_lazy("student_details")

    def get_success_url(self):
        url = self.get_redirect_url()
        return url or self.success_url



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import socket
import time

from django.core.urlresolvers import reverse

from deploy import captcha

# Uncomment to use FireFox
# master_browser = webdriver.Firefox()
from portal.tests.pageObjects.portal.home_page import HomePage
from .selenium_test_case import SeleniumTestCase


class BaseTest(SeleniumTestCase):
    @classmethod
    def setUpClass(cls):
        cls.orig_captcha_enabled = captcha.CAPTCHA_ENABLED
        captcha.CAPTCHA_ENABLED = False
        super(BaseTest, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        captcha.CAPTCHA_ENABLED = cls.orig_captcha_enabled
        super(BaseTest, cls).tearDownClass()

    def go_to_homepage(self):
        path = reverse("home")
        self._go_to_path(path)
        return HomePage(self.selenium)

    def _go_to_path(self, path):
        socket.setdefaulttimeout(20)
        attempts = 0
        while attempts <= 3:
            try:
                self.selenium.get(self.live_server_url + path)
            except socket.timeout:
                if attempts > 2:
                    raise
                time.sleep(10)
            else:
                break
            attempts += 1



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from aimmo.urls import HOMEPAGE_REGEX
from django.conf.urls import include, url
from django.core.urlresolvers import reverse_lazy
from django.views.generic import RedirectView
from django.views.generic.base import TemplateView
from django.views.i18n import javascript_catalog
from game.views.level import play_default_level
from two_factor.urls import urlpatterns as two_factor_urls
from two_factor.views import (
    BackupTokensView,
    DisableView,
    LoginView,
    ProfileView,
    QRGeneratorView,
    SetupCompleteView,
    SetupView,
)
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.core import urls as wagtail_urls
from wagtail.documents import urls as wagtaildocs_urls

from common.permissions import teacher_verified
from portal.views.about import about
from portal.views.admin import AdminLoginView, aggregated_data, schools_map
from portal.views.aimmo.home import AimmoHomeView
from portal.views.api import (
    InactiveUsersView,
    last_connected_since,
    number_users_per_country,
    registered_users,
)
from portal.views.email import send_new_users_report, verify_email
from portal.views.help_and_support import contact
from portal.views.home import (
    home,
    home_learning,
    logout_view,
    process_newsletter_form,
    register_view,
)
from portal.views.login import (
    IndependentStudentLoginView,
    StudentLoginView,
    TeacherLoginView,
    old_login_form_redirect,
)
from portal.views.organisation import (
    OrganisationFuzzyLookup,
    organisation_leave,
    organisation_manage,
)
from portal.views.play_aimmo import play_aimmo
from portal.views.play_landing_page import play_landing_page
from portal.views.play_rapid_router import play_rapid_router
from portal.views.privacy_policy import privacy_policy
from portal.views.registration import (
    password_reset_check_and_confirm,
    password_reset_done,
    student_password_reset,
    teacher_password_reset,
)
from portal.views.student.edit_account_details import (
    IndependentStudentEditAccountView,
    SchoolStudentEditAccountView,
    student_edit_account,
)
from portal.views.student.play import student_details, student_join_organisation
from portal.views.teach import teach
from portal.views.teacher.dashboard import (
    dashboard_manage,
    organisation_allow_join,
    organisation_deny_join,
    organisation_kick,
    organisation_toggle_admin,
    teacher_accept_student_request,
    teacher_disable_2FA,
    teacher_reject_student_request,
)
from portal.views.teacher.solutions_level_selector import levels
from portal.views.teacher.teach import (
    default_solution,
    invite_teacher,
    materials_viewer,
    teacher_class_password_reset,
    teacher_delete_class,
    teacher_delete_students,
    teacher_dismiss_students,
    teacher_edit_class,
    teacher_edit_student,
    teacher_move_class,
    teacher_move_students,
    teacher_move_students_to_class,
    teacher_onboarding_create_class,
    teacher_onboarding_edit_class,
    teacher_print_reminder_cards,
    teacher_student_reset,
    teacher_view_class,
)
from portal.views.teacher.teacher_materials import materials
from portal.views.teacher.teacher_resources import teacher_resources
from portal.views.terms import terms
from django.conf import settings
from django.conf.urls.static import static

js_info_dict = {"packages": ("conf.locale",)}

two_factor_patterns = [
    url(r"^account/login/$", LoginView.as_view(), name="login"),
    url(r"", include(two_factor_urls, "two_factor")),
    url(r"^account/two_factor/setup/$", SetupView.as_view(), name="setup"),
    url(r"^account/two_factor/qrcode/$", QRGeneratorView.as_view(), name="qr"),
    url(
        r"^account/two_factor/setup/complete/$",
        SetupCompleteView.as_view(),
        name="setup_complete",
    ),
    url(
        r"^account/two_factor/backup/tokens/$",
        teacher_verified(BackupTokensView.as_view()),
        name="backup_tokens",
    ),
    url(
        r"^account/two_factor/$",
        teacher_verified(ProfileView.as_view()),
        name="profile",
    ),
    url(
        r"^account/two_factor/disable/$",
        teacher_verified(DisableView.as_view()),
        name="disable",
    ),
]


urlpatterns = [
    # The first AIMMO URL renders the new Kurono home page. It uses the same regex so as to overwrite the default
    # home page in the AIMMO project.
    # The second AIMMO URL imports all the URLs from the AIMMO project.
    url(r"^kurono/$", AimmoHomeView.as_view(), name="kurono"),
    url(HOMEPAGE_REGEX, include("aimmo.urls")),
    url(
        r"^favicon\.ico$",
        RedirectView.as_view(url="/static/portal/img/favicon.ico", permanent=True),
    ),
    url(
        r"^administration/login/$",
        AdminLoginView.as_view(),
        name="administration_login",
    ),
    url(
        r"^admin/$",
        RedirectView.as_view(url=reverse_lazy("aggregated_data"), permanent=True),
    ),
    url(r"^admin/map/$", schools_map, name="map"),
    url(r"^admin/data/$", aggregated_data, name="aggregated_data"),
    url(r"^mail/weekly", send_new_users_report, name="send_new_users_report"),
    url(r"^users/inactive/", InactiveUsersView.as_view(), name="inactive_users"),
    url(
        r"^locked_out/$",
        TemplateView.as_view(template_name="portal/locked_out.html"),
        name="locked_out",
    ),
    url(r"^", include(two_factor_patterns, "two_factor")),
    url(r"^i18n/", include("django.conf.urls.i18n")),
    url(r"^jsi18n/$", javascript_catalog, js_info_dict),
    url(r"^teach/solutions_navigation/$", levels, name="teacher_level_solutions"),
    url(
        r"^teach/solutions_navigation/(?P<levelName>[A-Z0-9]+)/$",
        default_solution,
        name="default_solution",
    ),
    url(r"^(?P<levelName>[A-Z0-9]+)/$", play_default_level, name="play_default_level"),
    url(r"^$", home, name="home"),
    url(r"^home-learning", home_learning, name="home-learning"),
    url(r"^register_form", register_view, name="register"),
    url(
        r"^login/teacher/$",
        TeacherLoginView.as_view(),
        name="teacher_login",
    ),
    url(
        r"^login/student/$",
        StudentLoginView.as_view(),
        name="student_login",
    ),
    url(
        r"^login/independent/$",
        IndependentStudentLoginView.as_view(),
        name="independent_student_login",
    ),
    url(r"^login_form", old_login_form_redirect, name="old_login_form"),
    url(r"^logout/$", logout_view, name="logout_view"),
    url(r"^news_signup/$", process_newsletter_form, name="process_newsletter_form"),
    url(
        r"^verify_email/$",
        TemplateView.as_view(template_name="portal/email_verification_needed.html"),
        name="email_verification",
    ),
    url(r"^verify_email/(?P<token>[0-9a-f]+)/$", verify_email, name="verify_email"),
    url(
        r"^user/password/reset/student/$",
        student_password_reset,
        name="student_password_reset",
    ),
    url(
        r"^user/password/reset/teacher/$",
        teacher_password_reset,
        name="teacher_password_reset",
    ),
    url(
        r"^user/password/reset/done/$",
        password_reset_done,
        name="reset_password_email_sent",
    ),
    url(
        r"^user/password/reset/(?P<uidb64>[0-9A-Za-z]+)-(?P<token>.+)/$",
        password_reset_check_and_confirm,
        name="password_reset_check_and_confirm",
    ),
    url(
        r"^teacher/password/reset/complete/$",
        TemplateView.as_view(template_name="portal/reset_password_done.html"),
        name="password_reset_complete",
    ),
    url(r"^teach/$", teach, name="teach"),
    url(
        r"^teach/fuzzy_lookup/$",
        OrganisationFuzzyLookup.as_view(),
        name="organisation_fuzzy_lookup",
    ),
    url(
        r"^teach/onboarding-organisation/$",
        organisation_manage,
        name="onboarding-organisation",
    ),
    url(
        r"^teach/onboarding-classes",
        teacher_onboarding_create_class,
        name="onboarding-classes",
    ),
    url(
        r"^teach/onboarding-class/(?P<access_code>[A-Z0-9]+)/$",
        teacher_onboarding_edit_class,
        name="onboarding-class",
    ),
    url(
        r"^teach/onboarding-class/(?P<access_code>[A-Z0-9]+)/print_reminder_cards/$",
        teacher_print_reminder_cards,
        name="teacher_print_reminder_cards",
    ),
    url(
        r"^teach/onboarding-complete",
        TemplateView.as_view(template_name="portal/teach/onboarding_complete.html"),
        name="onboarding-complete",
    ),
    url(r"^teach/invite", invite_teacher, name="invite_teacher"),
    url(r"^play/$", play_landing_page, name="play"),
    url(r"^play/details/$", student_details, name="student_details"),
    url(r"^play/account/$", student_edit_account, name="student_edit_account"),
    url(
        r"^play/account/independent/$",
        IndependentStudentEditAccountView.as_view(),
        name="independent_edit_account",
    ),
    url(
        r"^play/account/school_student/$",
        SchoolStudentEditAccountView.as_view(),
        name="school_student_edit_account",
    ),
    url(r"^play/join/$", student_join_organisation, name="student_join_organisation"),
    url(r"^play/rapid-router/$", play_rapid_router, name="play_rapid_router"),
    url(r"^play/kurono/$", play_aimmo, name="play_aimmo"),
    url(r"^about", about, name="about"),
    url(r"^help/$", contact, name="help"),
    url(r"^terms", terms, name="terms"),
    url(r"^privacy-policy", privacy_policy, name="privacy_policy"),
    url(r"^teach/materials/$", materials, name="materials"),
    url(
        r"^teach/materials/(?P<pdf_name>[a-zA-Z0-9\/\-_]+)$",
        materials_viewer,
        name="materials_viewer",
    ),
    url(r"^teach/resources/$", teacher_resources, name="teaching_resources"),
    url(r"^teach/dashboard/$", dashboard_manage, name="dashboard"),
    url(
        r"^teach/dashboard/kick/(?P<pk>[0-9]+)/$",
        organisation_kick,
        name="organisation_kick",
    ),
    url(
        r"^teach/dashboard/toggle_admin/(?P<pk>[0-9]+)/$",
        organisation_toggle_admin,
        name="organisation_toggle_admin",
    ),
    url(
        r"^teach/dashboard/disable_2FA/(?P<pk>[0-9]+)/$",
        teacher_disable_2FA,
        name="teacher_disable_2FA",
    ),
    url(
        r"^teach/dashboard/allow_join/(?P<pk>[0-9]+)/$",
        organisation_allow_join,
        name="organisation_allow_join",
    ),
    url(
        r"^teach/dashboard/deny_join/(?P<pk>[0-9]+)/$",
        organisation_deny_join,
        name="organisation_deny_join",
    ),
    url(
        r"^teach/dashboard/school/leave/$",
        organisation_leave,
        name="organisation_leave",
    ),
    url(
        r"^teach/dashboard/student/accept/(?P<pk>[0-9]+)/$",
        teacher_accept_student_request,
        name="teacher_accept_student_request",
    ),
    url(
        r"^teach/dashboard/student/reject/(?P<pk>[0-9]+)/$",
        teacher_reject_student_request,
        name="teacher_reject_student_request",
    ),
    url(
        r"^teach/class/(?P<access_code>[A-Z0-9]+)/$",
        teacher_view_class,
        name="view_class",
    ),
    url(
        r"^teach/class/delete/(?P<access_code>[A-Z0-9]+)/$",
        teacher_delete_class,
        name="teacher_delete_class",
    ),
    url(
        r"^teach/class/(?P<access_code>[A-Z0-9]+)/students/delete/$",
        teacher_delete_students,
        name="teacher_delete_students",
    ),
    url(
        r"^teach/class/edit/(?P<access_code>[A-Z0-9]+)/$",
        teacher_edit_class,
        name="teacher_edit_class",
    ),
    url(
        r"^teach/class/student/edit/(?P<pk>[0-9]+)/$",
        teacher_edit_student,
        name="teacher_edit_student",
    ),
    url(
        r"^teach/class/student/reset/(?P<pk>[0-9]+)/$",
        teacher_student_reset,
        name="teacher_student_reset",
    ),
    url(
        r"^teach/class/(?P<access_code>[A-Z0-9]+)/password_reset/$",
        teacher_class_password_reset,
        name="teacher_class_password_reset",
    ),
    url(
        r"^teach/class/(?P<access_code>[A-Z0-9]+)/students/dismiss/$",
        teacher_dismiss_students,
        name="teacher_dismiss_students",
    ),
    url(
        r"^teach/class/move/(?P<access_code>[A-Z0-9]+)/$",
        teacher_move_class,
        name="teacher_move_class",
    ),
    url(
        r"^teach/class/(?P<access_code>[A-Z0-9]+)/students/move/$",
        teacher_move_students,
        name="teacher_move_students",
    ),
    url(
        r"^teach/class/(?P<access_code>[A-Z0-9]+)/students/move/disambiguate/$",
        teacher_move_students_to_class,
        name="teacher_move_students_to_class",
    ),
    url(
        r"^api/",
        include(
            [
                url(
                    r"^registered/(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/$",
                    registered_users,
                    name="registered-users",
                ),
                url(
                    r"^lastconnectedsince/(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/$",
                    last_connected_since,
                    name="last-connected-since",
                ),
                url(
                    r"^userspercountry/(?P<country>(AF|AX|AL|DZ|AS|AD|AO|AI|AQ|AG|AR|AM|AW|AU|AT|AZ|BS|BH|BD|BB|BY|BE|BZ|BJ|BM|BT|BO|BQ|BA|BW|BV|BR|IO|BN|BG|BF|BI|KH|CM|CA|CV|KY|CF|TD|CL|CN|CX|CC|CO|KM|CG|CD|CK|CR|CI|HR|CU|CW|CY|CZ|DK|DJ|DM|DO|EC|EG|SV|GQ|ER|EE|ET|FK|FO|FJ|FI|FR|GF|PF|TF|GA|GM|GE|DE|GH|GI|GR|GL|GD|GP|GU|GT|GG|GN|GW|GY|HT|HM|VA|HN|HK|HU|IS|IN|ID|IR|IQ|IE|IM|IL|IT|JM|JP|JE|JO|KZ|KE|KI|KP|KR|KW|KG|LA|LV|LB|LS|LR|LY|LI|LT|LU|MO|MK|MG|MW|MY|MV|ML|MT|MH|MQ|MR|MU|YT|MX|FM|MD|MC|MN|ME|MS|MA|MZ|MM|NA|NR|NP|NL|NC|NZ|NI|NE|NG|NU|NF|MP|NO|OM|PK|PW|PS|PA|PG|PY|PE|PH|PN|PL|PT|PR|QA|RE|RO|RU|RW|BL|SH|KN|LC|MF|PM|VC|WS|SM|ST|SA|SN|RS|SC|SL|SG|SX|SK|SI|SB|SO|ZA|GS|SS|ES|LK|SD|SR|SJ|SZ|SE|CH|SY|TW|TJ|TZ|TH|TL|TG|TK|TO|TT|TN|TR|TM|TC|TV|UG|UA|AE|GB|US|UM|UY|UZ|VU|VE|VN|VG|VI|WF|EH|YE|ZM|ZW))/$",
                    number_users_per_country,
                    name="number_users_per_country",
                ),
            ]
        ),
    ),
    url(r"^hijack/", include("hijack.urls", namespace="hijack")),
    url(r"^cms/", include(wagtailadmin_urls)),
    url(r"^documents/", include(wagtaildocs_urls)),
    url(r"^pages/", include(wagtail_urls)),
]


if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal.strings.terms import TERMS_BANNER


def terms(request):
    return render(request, "portal/terms.html", {"BANNER": TERMS_BANNER})



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django.test import TestCase

from portal.templatetags.table_tags import resource_sheets_table, lengthen_list
from portal.views.teacher.teach import get_links


class MaterialsTests(TestCase):
    def test_lengthen_list_when_length_is_bigger(self):
        li = [1, 2, 3, 4]
        length = 10
        result = lengthen_list(length, li)
        assert len(result) == 10

    def test_lengthen_list_when_length_is_same(self):
        li = [1, 2, 3, 4]
        length = 4
        result = lengthen_list(length, li)
        assert len(result) == 4

    def test_lengthen_list_when_length_is_smaller(self):
        li = [1, 2, 3, 4]
        length = 2
        result = lengthen_list(length, li)
        assert len(result) == 4

    def test_padding_resource_sheet_table(self):
        table = {"starting_session_index": 1, "content": [[1, 2], [1, 2, 3], []]}
        result = resource_sheets_table(table)
        assert result["table"] == [[1, 2, []], [1, 2, 3], [[], [], []]]

    def test_padding_resource_sheet_table_when_index_is_not_one(self):
        table = {"starting_session_index": 6, "content": [[1, 2], [1, 2, 3], []]}
        result = resource_sheets_table(table)
        assert result["table"] == [[1, 2, []], [1, 2, 3], [[], [], []]]

    def test_underscores_are_removed_from_pdf_link_titles(self):
        pdf_name = "KS1_session_3"
        result = get_links(pdf_name)
        assert result == [("KS1_S3_1", "KS1 S3 1")]



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2018, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from common.models import Class, Student, Teacher
from django.contrib import messages as messages
from django.contrib.auth import logout
from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.urlresolvers import reverse_lazy
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.views.decorators.http import require_POST
from two_factor.utils import devices_for_user

from common import email_messages
from portal.forms.organisation import OrganisationForm
from portal.forms.teach import (
    ClassCreationForm,
    TeacherAddExternalStudentForm,
    TeacherEditAccountForm,
)
from common.helpers.emails import NOTIFICATION_EMAIL, send_email, update_email
from common.helpers.generators import generate_access_code, get_random_username
from portal.helpers.location import lookup_coord
from portal.helpers.password import check_update_password
from common.permissions import logged_in_as_teacher
from common.utils import using_two_factor


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def dashboard_teacher_view(request, is_admin):
    teacher = request.user.new_teacher
    school = teacher.school

    coworkers = Teacher.objects.filter(school=school).order_by(
        "new_user__last_name", "new_user__first_name"
    )

    join_requests = Teacher.objects.filter(pending_join_request=school).order_by(
        "new_user__last_name", "new_user__first_name"
    )
    requests = Student.objects.filter(pending_class_request__teacher=teacher)

    update_school_form = OrganisationForm(user=request.user, current_school=school)
    update_school_form.fields["name"].initial = school.name
    update_school_form.fields["postcode"].initial = school.postcode
    update_school_form.fields["country"].initial = school.country

    create_class_form = ClassCreationForm()

    update_account_form = TeacherEditAccountForm(request.user)
    update_account_form.fields["title"].initial = teacher.title
    update_account_form.fields["first_name"].initial = request.user.first_name
    update_account_form.fields["last_name"].initial = request.user.last_name

    anchor = ""

    backup_tokens = check_backup_tokens(request)

    if request.method == "POST":
        if can_process_update_school_form(request, is_admin):
            anchor = "school-details"
            update_school_form = OrganisationForm(
                request.POST, user=request.user, current_school=school
            )
            anchor = process_update_school_form(request, school, anchor)

        elif "create_class" in request.POST:
            anchor = "new-class"
            create_class_form = ClassCreationForm(request.POST)
            if create_class_form.is_valid():
                created_class = create_class_new(create_class_form, teacher)
                messages.success(
                    request,
                    "The class '{className}' has been created successfully.".format(
                        className=created_class.name
                    ),
                )
                return HttpResponseRedirect(
                    reverse_lazy(
                        "view_class", kwargs={"access_code": created_class.access_code}
                    )
                )

        else:
            anchor = "account"
            update_account_form = TeacherEditAccountForm(request.user, request.POST)
            changing_email, new_email, anchor = process_update_account_form(
                request, teacher, anchor
            )
            if changing_email:
                logout(request)
                messages.success(
                    request,
                    "Your account details have been successfully changed. Your email "
                    "will be changed once you have verified it, until then you can "
                    "still log in with your old email.",
                )
                return render(
                    request,
                    "portal/email_verification_needed.html",
                    {"userprofile": teacher.user, "email": new_email},
                )

    classes = Class.objects.filter(teacher=teacher)

    return render(
        request,
        "portal/teach/dashboard.html",
        {
            "teacher": teacher,
            "classes": classes,
            "is_admin": is_admin,
            "coworkers": coworkers,
            "join_requests": join_requests,
            "requests": requests,
            "update_school_form": update_school_form,
            "create_class_form": create_class_form,
            "update_account_form": update_account_form,
            "anchor": anchor,
            "backup_tokens": backup_tokens,
        },
    )


def can_process_update_school_form(request, is_admin):
    return "update_school" in request.POST and is_admin


def check_backup_tokens(request):
    backup_tokens = 0
    # For teachers using 2FA, find out how many backup tokens they have
    if using_two_factor(request.user):
        try:
            backup_tokens = request.user.staticdevice_set.all()[0].token_set.count()
        except Exception:
            backup_tokens = 0

    return backup_tokens


def process_update_school_form(request, school, old_anchor):
    update_school_form = OrganisationForm(
        request.POST, user=request.user, current_school=school
    )
    if update_school_form.is_valid():
        data = update_school_form.cleaned_data
        name = data.get("name", "")
        postcode = data.get("postcode", "")
        country = data.get("country", "")

        school.name = name
        school.postcode = postcode
        school.country = country

        error, country, town, lat, lng = lookup_coord(postcode, country)
        school.town = town
        school.latitude = lat
        school.longitude = lng
        school.save()

        anchor = "#"

        messages.success(
            request,
            "You have updated the details for your school or club successfully.",
        )
    else:
        anchor = old_anchor

    return anchor


def create_class_new(form, teacher):
    classmate_progress = False
    if form.cleaned_data["classmate_progress"] == "True":
        classmate_progress = True
    klass = Class.objects.create(
        name=form.cleaned_data["class_name"],
        teacher=teacher,
        access_code=generate_access_code(),
        classmates_data_viewable=classmate_progress,
    )
    return klass


def process_update_account_form(request, teacher, old_anchor):
    update_account_form = TeacherEditAccountForm(request.user, request.POST)
    changing_email = False
    new_email = ""
    if update_account_form.is_valid():
        data = update_account_form.cleaned_data

        # check not default value for CharField
        check_update_password(update_account_form, teacher.new_user, request, data)

        teacher.title = data["title"]
        teacher.new_user.first_name = data["first_name"]
        teacher.new_user.last_name = data["last_name"]

        changing_email, new_email = update_email(teacher.new_user, request, data)

        teacher.save()
        teacher.new_user.save()

        anchor = "#"

        messages.success(
            request, "Your account details have been successfully changed."
        )
    else:
        anchor = old_anchor

    return changing_email, new_email, anchor


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def dashboard_manage(request):
    teacher = request.user.new_teacher

    if teacher.school:
        return dashboard_teacher_view(request, teacher.is_admin)
    else:
        return HttpResponseRedirect(reverse_lazy("onboarding-organisation"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_allow_join(request, pk):
    teacher = get_object_or_404(Teacher, id=pk)
    user = request.user.new_teacher

    # check user has authority to accept teacher
    if teacher.pending_join_request != user.school or not user.is_admin:
        raise Http404

    teacher.school = teacher.pending_join_request
    teacher.pending_join_request = None
    teacher.is_admin = False
    teacher.save()

    messages.success(request, "The teacher has been added to your school or club.")

    emailMessage = email_messages.joinRequestAcceptedEmail(request, teacher.school.name)
    send_email(
        NOTIFICATION_EMAIL,
        [teacher.new_user.email],
        emailMessage["subject"],
        emailMessage["message"],
    )

    return HttpResponseRedirect(reverse_lazy("dashboard"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_deny_join(request, pk):
    teacher = get_object_or_404(Teacher, id=pk)
    user = request.user.new_teacher

    # check user has authority to accept teacher
    if teacher.pending_join_request != user.school or not user.is_admin:
        raise Http404

    teacher.pending_join_request = None
    teacher.save()

    messages.success(
        request, "The request to join your school or club has been successfully denied."
    )

    emailMessage = email_messages.joinRequestDeniedEmail(
        request, request.user.new_teacher.school.name
    )
    send_email(
        NOTIFICATION_EMAIL,
        [teacher.new_user.email],
        emailMessage["subject"],
        emailMessage["message"],
    )

    return HttpResponseRedirect(reverse_lazy("dashboard"))


def check_teacher_is_authorised(teacher, user):
    if teacher == user or (teacher.school != user.school or not user.is_admin):
        raise Http404


@require_POST
@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_kick(request, pk):
    teacher = get_object_or_404(Teacher, id=pk)
    user = request.user.new_teacher

    check_teacher_is_authorised(teacher, user)

    classes = Class.objects.filter(teacher=teacher)
    for klass in classes:
        teacher_id = request.POST.get(klass.access_code, None)
        if teacher_id:
            new_teacher = get_object_or_404(Teacher, id=teacher_id)
            klass.teacher = new_teacher
            klass.save()

    classes = Class.objects.filter(teacher=teacher)
    teachers = Teacher.objects.filter(school=teacher.school).exclude(id=teacher.id)

    if classes.exists():
        messages.info(
            request,
            "This teacher still has classes assigned to them. You must first move them "
            "to another teacher in your school or club.",
        )
        return render(
            request,
            "portal/teach/teacher_move_all_classes.html",
            {
                "original_teacher": teacher,
                "classes": classes,
                "teachers": teachers,
                "submit_button_text": "Remove teacher",
            },
        )

    teacher.school = None
    teacher.save()

    messages.success(
        request, "The teacher has been successfully removed from your school or club.",
    )

    emailMessage = email_messages.kickedEmail(request, user.school.name)

    send_email(
        NOTIFICATION_EMAIL,
        [teacher.new_user.email],
        emailMessage["subject"],
        emailMessage["message"],
    )

    return HttpResponseRedirect(reverse_lazy("dashboard"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def organisation_toggle_admin(request, pk):
    teacher = get_object_or_404(Teacher, id=pk)
    user = request.user.new_teacher

    check_teacher_is_authorised(teacher, user)

    teacher.is_admin = not teacher.is_admin
    teacher.save()

    if teacher.is_admin:
        messages.success(request, "Administrator status has been given successfully.")
        emailMessage = email_messages.adminGivenEmail(request, teacher.school.name)
    else:
        messages.success(request, "Administrator status has been revoked successfully.")
        emailMessage = email_messages.adminRevokedEmail(request, teacher.school.name)

    send_email(
        NOTIFICATION_EMAIL,
        [teacher.new_user.email],
        emailMessage["subject"],
        emailMessage["message"],
    )

    return HttpResponseRedirect(reverse_lazy("dashboard"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_disable_2FA(request, pk):
    teacher = get_object_or_404(Teacher, id=pk)
    user = request.user.new_teacher

    # check user has authority to change
    if teacher.school != user.school or not user.is_admin:
        raise Http404

    [
        device.delete()
        for device in devices_for_user(teacher.new_user)
        if request.method == "POST"
    ]

    return HttpResponseRedirect(reverse_lazy("dashboard"))


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_accept_student_request(request, pk):
    student = get_object_or_404(Student, id=pk)
    teacher = request.user.new_teacher

    check_student_can_be_accepted(request, student)

    students = Student.objects.filter(
        class_field=student.pending_class_request
    ).order_by("new_user__first_name")

    if request.method == "POST":
        form = TeacherAddExternalStudentForm(
            student.pending_class_request, request.POST
        )
        if form.is_valid():
            data = form.cleaned_data
            student.class_field = student.pending_class_request
            student.pending_class_request = None
            student.new_user.username = get_random_username()
            student.new_user.first_name = data["name"]
            student.new_user.last_name = ""
            student.new_user.email = ""

            student.save()
            student.new_user.save()
            student.new_user.userprofile.save()

            return render(
                request,
                "portal/teach/teacher_added_external_student.html",
                {"student": student, "class": student.class_field},
            )
    else:
        form = TeacherAddExternalStudentForm(
            student.pending_class_request, initial={"name": student.new_user.first_name}
        )

    return render(
        request,
        "portal/teach/teacher_add_external_student.html",
        {
            "students": students,
            "class": student.pending_class_request,
            "student": student,
            "form": form,
        },
    )


def check_student_can_be_accepted(request, student):
    """
    Check student is awaiting decision on request
    """
    if not student.pending_class_request:
        raise Http404

    # check user (teacher) has authority to accept student
    if request.user.new_teacher != student.pending_class_request.teacher:
        raise Http404


@login_required(login_url=reverse_lazy("teacher_login"))
@user_passes_test(logged_in_as_teacher, login_url=reverse_lazy("teacher_login"))
def teacher_reject_student_request(request, pk):
    student = get_object_or_404(Student, id=pk)

    # check student is awaiting decision on request
    if not student.pending_class_request:
        raise Http404

    # check user (teacher) has authority to reject student
    if request.user.new_teacher != student.pending_class_request.teacher:
        raise Http404

    emailMessage = email_messages.studentJoinRequestRejectedEmail(
        request,
        student.pending_class_request.teacher.school.name,
        student.pending_class_request.access_code,
    )
    send_email(
        NOTIFICATION_EMAIL,
        [student.new_user.email],
        emailMessage["subject"],
        emailMessage["message"],
    )

    student.pending_class_request = None
    student.save()

    messages.success(
        request,
        "Request from external/independent student has been rejected successfully.",
    )

    return HttpResponseRedirect(reverse_lazy("dashboard"))



# coding=utf-8
PLAY_BANNER = {
    "title": "Play",
    "subtitle": "Anyone can learn how to code. We will help you learn how. It’s "
    "fun, free and easy.",
    "text": "",
    "image_class": "banner--picture--play",
}

PLAY_HEADLINE = {
    "title": "Anyone can code, you can too!",
    "description": "Whether you’re a teacher or a student, our games support and "
    "guide you, making learning to code great fun. Get started with Rapid Router "
    "designed for students new to coding. This is where you will build up your "
    "ability until you are ready to advance to Kurono, where you can test "
    "your skills in Python.",
}

PLAY_BENEFITS = {
    "first": {
        "image": "portal/img/teaching_tools.jpg",
        "title": "Free teaching tools",
        "text": "Free teaching resources, lesson plans and much more. You can even "
        "track your students’ progress with our class tracker.",
        "button": {"text": "", "link": ""},
    },
    "second": {
        "image": "portal/img/a_fun_game_for_students.jpg",
        "title": "Fun games for students",
        "text": "You can learn to become a coding superhero, amaze your friends "
        "with your high scores.",
        "button": {"text": "", "link": ""},
    },
    "third": {
        "image": "portal/img/get_creative.png",
        "title": "Get creative!",
        "text": "Work together to find multiple ways to solve challenges.",
        "button": {"text": "", "link": ""},
    },
}

RAPID_ROUTER_BANNER = {
    "title": "Rapid Router",
    "description": "Use Blockly (and later Python) to help Dee drive the van "
    "to make all the deliveries and become an Ocado delivery hero.",
    "button_text": "Find out more",
    "button_link": "play_rapid_router",
    "ages": "Ages 6—13",
    "background_image_class": "banner--rapid-router--background",
}

KURONO_BANNER = {
    "title": "Kurono",
    "description": "Using intermediate Python, join your class on a time "
    "travelling adventure. Work together to complete challenges and unlock "
    "new time periods.",
    "button_text": "Find out more",
    "button_link": "play_aimmo",
    "ages": "Ages 13+",
    "background_image_class": "banner--aimmo--background",
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from builtins import range

from django.core import mail

from common.tests.utils import email as email_utils
from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import generate_details, signup_teacher_directly
from portal.tests.pageObjects.portal.base_page import BasePage
from portal.tests.pageObjects.portal.home_page import HomePage

from .base_test import BaseTest
from .utils.messages import is_organisation_created_message_showing
from .utils.organisation import (
    create_organisation,
    create_organisation_directly,
    join_teacher_to_organisation,
)


class TestOrganisation(BaseTest, BasePage):
    def test_create(self):
        email, password = signup_teacher_directly()

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email, password)
        )

        page, name, postcode = create_organisation(page, password)
        assert is_organisation_created_message_showing(self.selenium, name)

    def test_join_empty(self):
        email, password = signup_teacher_directly()

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email, password)
            .join_empty_organisation()
        )

        assert page.__class__.__name__ == "OnboardingOrganisationPage"

    def test_create_clash(self):
        email_1, _ = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email_2, password_2)
            .create_organisation_failure(name, password_2, postcode)
        )

        assert page.has_creation_failed()

    def test_create_invalid_postcode(self):
        email, password = signup_teacher_directly()

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email, password)
        )

        page = page.create_organisation_failure("School", password, "   ")
        assert page.was_postcode_invalid()

    def test_revoke(self):
        email_1, _ = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email_2, password_2)
        )
        page = page.join_organisation(name)
        assert page.__class__.__name__ == "OnboardingRevokeRequestPage"
        assert page.check_organisation_name(name, postcode)

        page = page.revoke_join()
        assert page.__class__.__name__ == "OnboardingOrganisationPage"

    def test_allow_join(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)
        _, class_name, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email_2, password_2)
            .join_organisation(name)
        )

        assert page.__class__.__name__ == "OnboardingRevokeRequestPage"

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        assert page.has_join_request(email_2)
        page = page.accept_join_request()

        assert not page.has_join_request(email_2)

        page = (
            page.logout().go_to_teacher_login_page().login_no_class(email_2, password_2)
        )

        assert page.__class__.__name__ == "OnboardingClassesPage"

    def test_deny_join(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        name, _ = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email_2, password_2)
            .join_organisation(name)
        )

        assert page.__class__.__name__ == "OnboardingRevokeRequestPage"

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        assert page.has_join_request(email_2)
        page = page.deny_join_request()

        assert not page.has_join_request(email_2)

        page = (
            page.logout()
            .go_to_teacher_login_page()
            .login_no_school(email_2, password_2)
        )

        assert page.__class__.__name__ == "OnboardingOrganisationPage"

    def test_kick(self):
        email_1, password_1 = signup_teacher_directly()
        name, _ = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)

        title, first_name, _, email_2, password_2 = generate_details()

        new_last_name = "New Teacher"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_signup_page()
            .signup(title, first_name, new_last_name, email_2, password_2, password_2)
        )

        page = email_utils.follow_verify_email_link_to_onboarding(page, mail.outbox[0])
        mail.outbox = []

        page = page.login_no_school(email_2, password_2).join_organisation(name)

        assert page.__class__.__name__ == "OnboardingRevokeRequestPage"

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        page = page.accept_join_request()

        assert page.is_teacher_in_school(new_last_name)

        page = page.click_kick_button()
        assert page.is_dialog_showing()
        page = page.confirm_dialog()

        assert page.is_not_teacher_in_school(new_last_name)

    def test_move_classes_and_kick(self):
        email_1, password_1 = signup_teacher_directly()
        name, _ = create_organisation_directly(email_1)
        _, _, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)

        title, first_name, _, email_2, password_2 = generate_details()

        new_last_name = "New Teacher"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_signup_page()
            .signup(title, first_name, new_last_name, email_2, password_2, password_2)
        )

        page = email_utils.follow_verify_email_link_to_onboarding(page, mail.outbox[0])
        mail.outbox = []

        page = page.login_no_school(email_2, password_2).join_organisation(name)

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        page = page.accept_join_request()

        assert page.is_teacher_in_school(new_last_name)

        _, _, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        page = page.click_kick_button()
        assert page.is_dialog_showing()
        page = page.confirm_kick_with_students_dialog()

        assert page.__class__.__name__ == "TeachMoveClassesPage"

        page = page.move_and_kick()

        assert page.is_not_teacher_in_school(new_last_name)

    def test_leave_organisation(self):
        email_1, password_1 = signup_teacher_directly()
        name, _ = create_organisation_directly(email_1)
        _, _, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)

        title, first_name, _, email_2, password_2 = generate_details()

        new_last_name = "New Teacher"

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_signup_page()
            .signup(title, first_name, new_last_name, email_2, password_2, password_2)
        )

        page = email_utils.follow_verify_email_link_to_onboarding(page, mail.outbox[0])
        mail.outbox = []

        page = page.login_no_school(email_2, password_2).join_organisation(name)

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        page = page.accept_join_request()

        assert page.is_teacher_in_school(new_last_name)

        _, class_name_2, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        page = page.logout().go_to_teacher_login_page().login(email_2, password_2)

        page = page.leave_organisation_with_students()

        assert page.__class__.__name__ == "TeachMoveClassesPage"

        page = page.move_and_leave()

        assert page.__class__.__name__ == "OnboardingOrganisationPage"

        page = page.logout().go_to_teacher_login_page().login(email_1, password_1)

        assert page.is_not_teacher_in_school(new_last_name)

    def test_toggle_admin(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, _ = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)
        join_teacher_to_organisation(email_2, name, postcode)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email_1, password_1)
        )

        assert page.__class__.__name__ == "TeachDashboardPage"

        page = page.click_make_admin_button()
        assert page.is_dialog_showing()
        page = page.confirm_dialog()

        assert page.is_teacher_admin()

        page = page.click_make_non_admin_button()

        assert page.is_teacher_non_admin()

    def test_disable_2FA(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, _ = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)
        _, _, access_code = create_class_directly(email_1)
        create_school_student_directly(access_code)
        join_teacher_to_organisation(email_2, name, postcode)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email_1, password_1)
        )

        assert page.__class__.__name__ == "TeachDashboardPage"

        page = page.click_make_admin_button()
        assert page.is_dialog_showing()
        page = page.confirm_dialog()

        assert page.is_teacher_admin()

        page = page.click_make_non_admin_button()

        assert page.is_teacher_non_admin()

    def test_multiple_schools(self):
        # There was a bug where join requests to school 35 say would go to school 3,
        # 62 would go to 6, etc... this test checks for that

        n = 12

        emails, passwords, names, postcodes = self.initialise_data(n)

        for i in range(n):
            emails[i], passwords[i] = signup_teacher_directly()
            names[i], postcodes[i] = create_organisation_directly(emails[i])

        email, password = signup_teacher_directly()

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login_no_school(email, password)
        )

        page = page.join_organisation(names[n - 1])
        assert page.__class__.__name__ == "OnboardingRevokeRequestPage"
        assert page.check_organisation_name(names[n - 1], postcodes[n - 1])

    def initialise_data(self, n):
        emails = ["" for i in range(n)]
        passwords = ["" for i in range(n)]
        names = ["" for i in range(n)]
        postcodes = ["" for i in range(n)]

        return emails, passwords, names, postcodes

    def test_edit_details(self):
        email, password = signup_teacher_directly()
        school_name, postcode = create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        _, password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = HomePage(self.selenium).go_to_teacher_login_page().login(email, password)

        assert page.check_organisation_details(
            {"name": school_name, "postcode": postcode}
        )

        new_name = "new " + school_name
        new_postcode = "OX2 6LE"

        page.change_organisation_details({"name": new_name, "postcode": new_postcode})
        assert page.check_organisation_details(
            {"name": new_name, "postcode": new_postcode}
        )

    def test_edit_clash(self):
        email_1, _ = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        school_name_1, postcode_1 = create_organisation_directly(email_1)
        create_organisation_directly(email_2)
        _, _, access_code_1 = create_class_directly(email_1)
        _, _, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_1)
        create_school_student_directly(access_code_2)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_teacher_login_page()
            .login(email_2, password_2)
        )

        assert not page.check_organisation_details(
            {"name": school_name_1, "postcode": postcode_1}
        )

        page = page.change_organisation_details(
            {"name": school_name_1, "postcode": postcode_1}
        )

        assert page.has_edit_failed()



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from aimmo.templatetags.players_utils import get_user_playable_games
from django import template
from django.conf import settings
from django.template.defaultfilters import stringfilter
from portal import __version__
from portal import beta
from common.utils import using_two_factor

register = template.Library()


@register.filter(name="emaildomain")
@stringfilter
def emaildomain(email):
    return "*********" + email[email.find("@") :]


@register.filter(name="has_2FA")
def has_2FA(u):
    return using_two_factor(u)


@register.filter(name="is_logged_in")
def is_logged_in(u):
    return (
        u
        and u.is_authenticated()
        and (not using_two_factor(u) or (hasattr(u, "is_verified") and u.is_verified()))
    )


@register.filter
def is_developer(u):
    return not u.is_anonymous() and u.userprofile.developer


@register.filter
def has_beta_access(request):
    return beta.has_beta_access(request)


@register.inclusion_tag("portal/partials/aimmo_games_table.html", takes_context=True)
def games_table(context, base_url):
    playable_games = get_user_playable_games(context, base_url)
    playable_games["can_delete_game"] = True

    user = context.request.user
    if (
        hasattr(user, "userprofile")
        and hasattr(user.userprofile, "student")
        and user.userprofile.student.class_field != None
    ):
        playable_games["can_delete_game"] = False

    return playable_games


@register.filter(name="make_into_username")
def make_into_username(u):
    username = ""
    if hasattr(u, "userprofile"):
        if hasattr(u.userprofile, "student"):
            username = u.first_name
        elif hasattr(u.userprofile, "teacher"):
            username = u.userprofile.teacher.title + " " + u.last_name

    return username


@register.filter(name="is_logged_in_as_teacher")
def is_logged_in_as_teacher(u):
    return is_logged_in(u) and u.userprofile and hasattr(u.userprofile, "teacher")


@register.filter(name="is_independent_student")
def is_independent_student(u):
    return (
        u.userprofile
        and u.userprofile.student
        and u.userprofile.student.is_independent()
    )


@register.filter(name="has_teacher_finished_onboarding")
def has_teacher_finished_onboarding(u):
    teacher = u.userprofile.teacher
    classes = teacher.class_teacher.all()
    return (
        is_logged_in_as_teacher(u)
        and teacher.has_school()
        and classes
        and (classes.count() > 1 or classes[0].has_students())
    )


@register.filter(name="is_logged_in_as_school_user")
def is_logged_in_as_school_user(u):
    return (
        is_logged_in(u)
        and u.userprofile
        and (
            (
                hasattr(u.userprofile, "student")
                and u.userprofile.student.class_field != None
            )
            or hasattr(u.userprofile, "teacher")
        )
    )


@register.filter(name="get_user_status")
def get_user_status(u):
    if is_logged_in_as_school_user(u):
        if is_logged_in_as_teacher(u):
            return "TEACHER"
        else:
            return "SCHOOL_STUDENT"
    elif is_logged_in(u):
        return "INDEPENDENT_STUDENT"
    else:
        return "UNTRACKED"


@register.filter(name="make_title_caps")
def make_title_caps(s):
    if len(s) > 0:
        s = s[0].upper() + s[1:]
    return s


@register.filter(name="cloud_storage")
@stringfilter
def cloud_storage(e):
    return settings.CLOUD_STORAGE_PREFIX + e


@register.filter(name="get_project_version")
def get_project_version():
    return __version__



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

import time

from selenium.webdriver.support.ui import Select

from . import onboarding_classes_page
from . import onboarding_revoke_request_page
from .teach_base_page import TeachBasePage


class OnboardingOrganisationPage(TeachBasePage):
    def __init__(self, browser):
        super(OnboardingOrganisationPage, self).__init__(browser)

        assert self.on_correct_page("onboarding_organisation_page")

    def create_organisation(self, name, password, postcode, country="GB"):
        self._create_organisation(name, password, postcode, country)

        return onboarding_classes_page.OnboardingClassesPage(self.browser)

    def create_organisation_failure(self, name, password, postcode, country="GB"):
        self._create_organisation(name, password, postcode, country)

        return self

    def create_organisation_empty(self):
        self._click_create_school_button()

        return self

    def join_empty_organisation(self):
        self.browser.find_element_by_id("join-tab").click()
        self._click_join_school_button()

        return self

    def _create_organisation(self, name, password, postcode, country):
        self.browser.find_element_by_id("id_name").send_keys(name)
        self.browser.find_element_by_id("id_postcode").send_keys(postcode)
        country_element = self.browser.find_element_by_id("id_country")
        select = Select(country_element)
        select.select_by_value(country)
        self._click_create_school_button()

    def _click_create_school_button(self):
        self.browser.find_element_by_name("create_organisation").click()

    def _click_join_school_button(self):
        self.browser.find_element_by_name("join_organisation").click()

    def has_creation_failed(self):
        if not self.element_exists_by_css(".errorlist"):
            return False

        errors = (
            self.browser.find_element_by_id("form-create-organisation")
            .find_element_by_class_name("errorlist")
            .text
        )
        error = (
            "There is already a school or club registered with that name and postcode"
        )
        return error in errors

    def was_postcode_invalid(self):
        errors = (
            self.browser.find_element_by_id("form-create-organisation")
            .find_element_by_class_name("errorlist")
            .text
        )
        error = "Please enter a valid postcode or ZIP code"
        return error in errors

    def join_organisation(self, name):
        self.browser.find_element_by_id("join-tab").click()
        self.browser.find_element_by_id("id_fuzzy_name").send_keys(name)
        time.sleep(1)
        self._click_join_school_button()

        if self.on_correct_page("onboarding_revoke_request_page"):
            return onboarding_revoke_request_page.OnboardingRevokeRequestPage(
                self.browser
            )
        else:
            return self



from .base_page import BasePage
from .play import dashboard_page
from .student_password_reset_form_page import StudentPasswordResetFormPage
from .teach import dashboard_page as teach_dashboard_page
from .teach import onboarding_classes_page
from .teach import onboarding_students_page
from .teacher_password_reset_form_page import TeacherPasswordResetFormPage
import time


class TeacherLoginPage(BasePage):
    def __init__(self, browser):
        super(TeacherLoginPage, self).__init__(browser)

        assert self.on_correct_page("teacher_login_page")

    def login(self, email, password):
        self._login(email, password)
        return teach_dashboard_page.TeachDashboardPage(self.browser)

    def _login(self, email, password):
        self.browser.find_element_by_id("id_username").send_keys(email)
        self.browser.find_element_by_id("id_password").send_keys(password)
        self.browser.find_element_by_name("login_view").click()

    def login_no_school(self, email, password):
        self._login(email, password)
        import portal.tests.pageObjects.portal.teach.onboarding_organisation_page as onboarding_organisation_page

        return onboarding_organisation_page.OnboardingOrganisationPage(self.browser)

    def login_no_class(self, email, password):
        self._login(email, password)

        return onboarding_classes_page.OnboardingClassesPage(self.browser)

    def login_no_students(self, email, password):
        self._login(email, password)

        return onboarding_students_page.OnboardingStudentsPage(self.browser)

    def login_failure(self, email, password):
        self._login(email, password)
        return self

    def has_login_failed(self, form_id, error):
        time.sleep(1.5)
        errors = (
            self.browser.find_element_by_id(form_id)
            .find_element_by_class_name("errorlist")
            .text
        )
        return error in errors

    def go_to_teacher_forgotten_password_page(self):
        self.browser.find_element_by_id("teacher_forgotten_password_button").click()
        return TeacherPasswordResetFormPage(self.browser)



# -*- coding: utf-8 -*-
import re
import sys

from setuptools import find_packages, setup

with open("portal/__init__.py", "r") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

try:
    from semantic_release import setup_hook

    setup_hook(sys.argv)
except ImportError:
    pass

setup(
    name="codeforlife-portal",
    version=version,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "django>=1.10.8, <= 1.11.24",
        "django-countries==5.4",
        "djangorestframework>=3.8.2, < 3.9.0",
        "django-autoconfig==0.8.0",
        "django-pipeline==1.6.14",
        "django-recaptcha==2.0.5",
        "pyyaml==4.2b1",
        "rapid-router >= 1.0.0.post.dev1",
        "aimmo",
        "reportlab>=3.5.32,<3.6.0",
        "django-formtools==2.1",
        "django-otp<=0.7.0",  # we needed to fix this due to a wide ranged dependency in django-two-factor-auth
        "requests>=2.22.0,<2.23.0",
        "django-treebeard==4.3",
        "django-sekizai==1.0.0",
        "django-classy-tags==1.*",
        "Pillow==5.4.1",
        "sqlparse",
        "libsass",
        "phonenumbers>=8.11.0, <8.12.0",
        "more-itertools==5.0.0",  # 8.0.2 doesn't support Python <=3.4
        "future",
        "django-hijack>=2.1.10, <2.2.0",
        "django-hijack-admin>=2.1.10, <2.2.0",
        f"cfl-common=={version}",
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Framework :: Django",
    ],
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import signup_teacher_directly
from portal.tests.utils.classes import create_class, transfer_class

from .base_test import BaseTest
from .utils.messages import (
    is_class_created_message_showing,
    is_class_nonempty_message_showing,
)
from .utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)


class TestClass(BaseTest):
    def test_create(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        page = (
            self.go_to_homepage()
            .go_to_teacher_login_page()
            .login_no_class(email, password)
        )

        assert page.does_not_have_classes()

        page, class_name = create_class(page)
        assert is_class_created_message_showing(self.selenium, class_name)

    def test_create_dashboard(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        klass, name, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.go_to_homepage().go_to_teacher_login_page().login(email, password)

        page, class_name = create_class(page)

        assert is_class_created_message_showing(self.selenium, class_name)

    def test_create_dashboard_non_admin(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        name, postcode = create_organisation_directly(email_1)
        klass_1, class_name_1, access_code_1 = create_class_directly(email_1)
        create_school_student_directly(access_code_1)
        join_teacher_to_organisation(email_2, name, postcode)
        klass_2, class_name_2, access_code_2 = create_class_directly(email_2)
        create_school_student_directly(access_code_2)

        page = (
            self.go_to_homepage().go_to_teacher_login_page().login(email_2, password_2)
        )

        page, class_name_3 = create_class(page)

        assert is_class_created_message_showing(self.selenium, class_name_3)

    def test_delete_empty(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, class_name, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.go_to_homepage().go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page()

        page = page.toggle_select_student().delete_students()
        assert page.is_dialog_showing()
        page = page.confirm_dialog()
        page = page.delete_class()
        assert page.is_dialog_showing()
        page = page.confirm_delete_class_dialog()
        assert page.__class__.__name__ == "TeachDashboardPage"
        assert page.does_not_have_classes()

    def test_delete_nonempty(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, class_name, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.go_to_homepage().go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page()

        page = page.delete_class()
        assert page.is_dialog_showing()
        page = page.cancel_dialog()
        page = page.delete_class()
        assert page.is_dialog_showing()
        page = page.confirm_dialog_expect_error()
        assert page.__class__.__name__ == "TeachClassPage"
        page.wait_for_messages()
        assert is_class_nonempty_message_showing(self.selenium)

    def test_edit(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, class_name, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.go_to_homepage().go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page().go_to_class_settings_page()

        new_class_name = "new " + class_name
        assert not page.check_class_details(
            {"name": new_class_name, "classmates_data_viewable": True}
        )

        page = page.change_class_details(
            {"name": new_class_name, "classmates_data_viewable": True}
        )

        page = page.go_to_class_settings_page()
        new_class_name = "new " + class_name
        assert page.check_class_details(
            {"name": new_class_name, "classmates_data_viewable": True}
        )

    def test_transfer_cancel(self):
        email, password = signup_teacher_directly()
        create_organisation_directly(email)
        _, class_name, access_code = create_class_directly(email)
        create_school_student_directly(access_code)

        page = self.go_to_homepage().go_to_teacher_login_page().login(email, password)
        page = page.go_to_class_page().go_to_class_settings_page()

        page = page.transfer_class()
        assert page.get_list_length() == 0
        page = page.cancel()
        assert page.__class__.__name__ == "TeachClassPage"

    def test_transfer(self):
        email_1, password_1 = signup_teacher_directly()
        email_2, password_2 = signup_teacher_directly()
        org_name, postcode = create_organisation_directly(email_1)
        join_teacher_to_organisation(email_2, org_name, postcode)
        _, class_name, access_code = create_class_directly(email_1)
        student_name, student_password, _ = create_school_student_directly(access_code)

        page = (
            self.go_to_homepage().go_to_teacher_login_page().login(email_1, password_1)
        )
        page = page.go_to_class_page().go_to_class_settings_page()

        page = transfer_class(page, 0)
        assert page.does_not_have_classes()

        page = page.logout().go_to_teacher_login_page().login(email_2, password_2)
        assert page.does_class_exist(class_name, access_code)
        page = page.go_to_class_page()
        assert page.has_students()
        assert page.student_exists(student_name)



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal.strings.teach import TEACH_BANNER, TEACH_BENEFITS


def teach(request):
    return render(
        request,
        "portal/teach.html",
        {"BANNER": TEACH_BANNER, "BENEFITS": TEACH_BENEFITS},
    )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.

from django.contrib import messages as messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.core.urlresolvers import reverse_lazy
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import render

from common import email_messages
from portal.forms.play import StudentJoinOrganisationForm
from common.helpers.emails import send_email, NOTIFICATION_EMAIL
from common.permissions import logged_in_as_student, logged_in_as_independent_student


@login_required(login_url=reverse_lazy("student_login"))
@user_passes_test(logged_in_as_student, login_url=reverse_lazy("student_login"))
def student_details(request):
    return render(request, "portal/play/student_details.html")


def username_labeller(request):
    return request.user.username


@login_required(login_url=reverse_lazy("independent_student_login"))
@user_passes_test(
    logged_in_as_independent_student,
    login_url=reverse_lazy("independent_student_login"),
)
def student_join_organisation(request):

    student = request.user.new_student
    request_form = StudentJoinOrganisationForm()

    # check student not managed by a school
    if student.class_field:
        return HttpResponseRedirect(reverse_lazy("student_details"))

    if request.method == "POST":
        if "class_join_request" in request.POST:
            request_form = StudentJoinOrganisationForm(request.POST)
            process_join_organisation_form(request_form, request, student)

        elif "revoke_join_request" in request.POST:
            student.pending_class_request = None
            student.save()
            # Check teacher hasn't since accepted rejection before posting success
            show_cancellation_message_if_student_not_in_class(student, request)
            return HttpResponseRedirect(reverse_lazy("student_edit_account"))

    res = render(
        request,
        "portal/play/student_join_organisation.html",
        {"request_form": request_form, "student": student},
    )
    return res


def process_join_organisation_form(request_form, request, student):
    if request_form.is_valid():
        student.pending_class_request = request_form.klass
        student.save()

        email_message = email_messages.studentJoinRequestSentEmail(
            request,
            request_form.klass.teacher.school.name,
            request_form.klass.access_code,
        )
        send_email(
            NOTIFICATION_EMAIL,
            [student.new_user.email],
            email_message["subject"],
            email_message["message"],
        )

        email_message = email_messages.studentJoinRequestNotifyEmail(
            request,
            student.new_user.username,
            student.new_user.email,
            student.pending_class_request.access_code,
        )
        send_email(
            NOTIFICATION_EMAIL,
            [student.pending_class_request.teacher.new_user.email],
            email_message["subject"],
            email_message["message"],
        )

        messages.success(
            request, "Your request to join a school has been received successfully."
        )


def show_cancellation_message_if_student_not_in_class(student, request):
    if not student.class_field:
        messages.success(
            request, "Your request to join a school has been cancelled successfully."
        )



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from __future__ import absolute_import

from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import signup_teacher_directly
from portal.tests.pageObjects.portal.home_page import HomePage

from .base_test import BaseTest
from .utils.messages import is_student_details_updated_message_showing
from .utils.organisation import create_organisation_directly


class TestSchoolStudent(BaseTest):
    def test_login(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login(student_name, access_code, student_password)
        )
        assert self.is_dashboard(page)

    def test_login_failure(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, _, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login_failure(student_name, access_code, "some other password")
        )

        assert page.has_login_failed(
            "form-login-school", "Invalid name, class access code or password"
        )

    def test_login_nonexistent_class(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login_failure(student_name, "WRONG", student_password)
        )

        assert page.has_login_failed(
            "form-login-school", "Invalid name, class access code or password"
        )

    def test_login_empty_class(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)
        _, _, access_code2 = create_class_directly(email)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login_failure(student_name, access_code2, student_password)
        )

        assert page.has_login_failed(
            "form-login-school", "Invalid name, class access code or password"
        )

    def test_update_password_current_password_wrong(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login(student_name, access_code, student_password)
        )
        assert self.is_dashboard(page)

        page = page.go_to_account_page().update_password_failure(
            "NewPassword", "NewPassword", "WrongPassword"
        )
        assert self.is_account_page(page)
        assert page.was_form_invalid(
            "student_account_form", "Your current password was incorrect"
        )

    def test_update_password_passwords_not_match(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login(student_name, access_code, student_password)
        )
        assert self.is_dashboard(page)

        page = page.go_to_account_page().update_password_failure(
            "NewPassword1", "OtherPassword1", student_password
        )
        assert self.is_account_page(page)
        assert page.was_form_invalid(
            "student_account_form", "Your new passwords do not match"
        )

    def test_update_password_too_weak(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login(student_name, access_code, student_password)
        )
        assert self.is_dashboard(page)

        page = page.go_to_account_page().update_password_failure(
            "tiny", "tiny", student_password
        )
        assert self.is_account_page(page)
        assert page.was_form_invalid(
            "student_account_form",
            "Password not strong enough, consider using at least 8 characters, upper and lower case letters, and numbers",
        )

    def test_update_password_success(self):
        email, _ = signup_teacher_directly()
        create_organisation_directly(email)
        _, _, access_code = create_class_directly(email)
        student_name, student_password, _ = create_school_student_directly(access_code)

        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login(student_name, access_code, student_password)
        )
        assert self.is_dashboard(page)

        new_password = "NewPassword1"

        page = page.go_to_account_page().update_password_failure(
            new_password, new_password, student_password
        )
        assert is_student_details_updated_message_showing(self.selenium)

        page.logout()
        self.selenium.get(self.live_server_url)
        page = (
            HomePage(self.selenium)
            .go_to_student_login_page()
            .student_login(student_name, access_code, new_password)
        )
        assert self.is_dashboard(page)

    def is_dashboard(self, page):
        return page.__class__.__name__ == "PlayDashboardPage"

    def is_account_page(self, page):
        return page.__class__.__name__ == "PlayAccountPage"



from django import template

register = template.Library()


@register.inclusion_tag("portal/partials/hero_card.html", takes_context=True)
def hero_card(context):
    """
    Registers the inclusion tag for the hero card partial.
    The template currently expects the following context elements:
    - image: the path to the image at the top of the hero card
    - title: the heading of the hero card
    - description: the text paragraph of the hero card
    - button1_text: the text on the card's first button
    - button1_link: the link that the first button redirects to
    - button2_text: the text on the card's second button
    - button2_link: the link that the second button redirects to
    """
    return context["HERO_CARD"]



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Innovation Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
import json
import os
import unittest

import requests
import responses

from portal.helpers.location import lookup_coord, lookup_country

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")

# Returns a string read from the given file that can be deserialised into json objects
def read_json_from_file(filename):
    data = ""

    if filename:
        f = open(filename, "r")
        data = f.read()
        f.close()

    return data


# Returns path to the file that contains the json response
def datafile(filename):
    return os.path.join(DATA_DIR, filename)


MAPS_API_GEOCODE_JSON = "https://maps.googleapis.com/maps/api/geocode/json?"


class TestLocation(unittest.TestCase):
    def assert_default_coord(self, town, lat, lng):
        # default values returned when error occurs in lookup_coord()
        self.assertEqual((town, lat, lng), (0, 0, 0))

    def assert_default_country_and_coord(self, country, town, lat, lng):
        # default values returned when error occurs in lookup_country()
        self.assertEqual((country, town, lat, lng), ("GB", 0, 55.378051, -3.435973))

    @responses.activate
    def test_lookup_coord_call_api_once(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=SW72AZ&components=country:GB",
            body=read_json_from_file(datafile("sw72az_gb.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_coord("SW72AZ", "GB")
        self.assertEqual(len(responses.calls), 1, "API was called more than once")

    @responses.activate
    def test_lookup_coord_json_unchanged(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=SW72AZ&components=country:GB",
            body=read_json_from_file(datafile("sw72az_gb.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_coord("SW72AZ", "GB")
        self.assertEqual(
            responses.calls[0].response.json(),
            json.loads(read_json_from_file(datafile("sw72az_gb.json"))),
        )

    @responses.activate
    def test_lookup_coord_valid_postcode_country_gb1(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=SW72AZ&components=country:GB",
            body=read_json_from_file(datafile("sw72az_gb.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_coord("SW72AZ", "GB")
        self.assertEqual(result, (None, "GB", "London", 51.5005046, -0.1782187))

    @responses.activate
    def test_lookup_coord_valid_postcode_country_gb2(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=AL10 9NE&components=country:GB",
            body=read_json_from_file(datafile("al109ne_gb.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_coord("AL10 9NE", "GB")
        self.assertEqual(result, (None, "GB", "Hatfield", 51.7623259, -0.2438929))

    # Default to coordinates of country
    @responses.activate
    def test_lookup_coord_invalid_postcode_country_gb(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=10000&components=country:GB",
            body=read_json_from_file(datafile("10000_gb.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_coord("10000", "GB")
        self.assertEqual(result, (None, "GB", 0, 55.378051, -3.435973))

    @responses.activate
    def test_lookup_coord_invalid_postcode_country_kr(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=AL109NE&components=country:KR",
            body=read_json_from_file(datafile("al109ne_kr.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_coord("AL109NE", "KR")
        self.assertEqual(result, (None, "KR", 0, 35.907757, 127.766922))

    @responses.activate
    def test_lookup_coord_connection_error(self):
        error, country, town, lat, lng = lookup_coord("AL109NE", "GB")
        assert "Connection error" in error
        self.assert_default_coord(town, lat, lng)

    @responses.activate
    def test_lookup_coord_cannot_decode_json(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=AL109NE&components=country:GB",
            body="",
            match_querystring=True,
            content_type="application/json",
        )
        error, country, town, lat, lng = lookup_coord("AL109NE", "GB")
        assert "Value error" in error
        self.assert_default_coord(town, lat, lng)

    @responses.activate
    def test_lookup_coord_request_error_404(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=AL109NE&components=country:GB",
            body=read_json_from_file(datafile("al109ne_gb.json")),
            status=requests.codes.not_found,
            match_querystring=True,
            content_type="application/json",
        )
        error, country, town, lat, lng = lookup_coord("AL109NE", "GB")
        assert "Request error" in error
        self.assert_default_coord(town, lat, lng)

    @responses.activate
    def test_lookup_coord_zero_results(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "address=AL109NE&components=country:AB",
            body=read_json_from_file(datafile("al109ne_ab.json")),
            match_querystring=True,
            content_type="application/json",
        )
        error, country, town, lat, lng = lookup_coord("AL109NE", "AB")
        assert "API error" in error
        self.assert_default_coord(town, lat, lng)

    @responses.activate
    def test_lookup_country_valid_postcode1(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "components=postal_code:SW72AZ",
            body=read_json_from_file(datafile("sw72az.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_country("SW72AZ")
        self.assertEqual(result, (None, "GB", "London", 51.5005046, -0.1782187))

    @responses.activate
    def test_lookup_country_valid_postcode1(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "components=postal_code:AL10 9NE",
            body=read_json_from_file(datafile("al109ne.json")),
            match_querystring=True,
            content_type="application/json",
        )
        result = lookup_country("AL10 9NE")
        self.assertEqual(result, (None, "GB", "Hatfield", 51.7623259, -0.2438929))

    @responses.activate
    def test_lookup_country_invalid_postcode_zero_results(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "components=postal_code:xxxxx",
            body=read_json_from_file(datafile("xxxxx.json")),
            match_querystring=True,
            content_type="application/json",
        )
        error, country, town, lat, lng = lookup_country("xxxxx")
        assert "API error" in error
        self.assert_default_country_and_coord(country, town, lat, lng)

    @responses.activate
    def test_lookup_country_connection_error(self):
        error, country, town, lat, lng = lookup_country("xxxxx")
        assert "Connection error" in error

    @responses.activate
    def test_lookup_country_request_error_404(self):
        responses.add(
            responses.GET,
            MAPS_API_GEOCODE_JSON + "components=postal_code:AL109NE",
            body=read_json_from_file(datafile("al109ne.json")),
            status=requests.codes.not_found,
            match_querystring=True,
            content_type="application/json",
        )
        error, country, town, lat, lng = lookup_country("AL109NE")
        assert "Request error" in error
        self.assert_default_country_and_coord(country, town, lat, lng)



# -*- coding: utf-8 -*-
from django.shortcuts import render

from portal.strings.privacy_policy import PRIVACY_POLICY_BANNER


def privacy_policy(request):
    return render(
        request, "portal/privacy_policy.html", {"BANNER": PRIVACY_POLICY_BANNER}
    )



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots[
    "test_banner 1"
] = """<div class="banner banner--teacher row">
    <div class="col-sm-5 col-sm-offset-2">
        <h1 class="banner__text--primary">Test title</h1>
        
            <h4>Test subtitle</h4>
        
        
            <p>Test text</p>
        
    </div>
    <div class="col-sm-5">
        <div class="banner--picture"><div class="banner--picture__inside1"><div class="banner--picture__inside2 test--image--class"></div></div></div>
    </div>
</div>
"""

snapshots[
    "test_headline 1"
] = """<h1>Test title</h1>
<h6 class="col-sm-6 col-center">Test description</h6>
"""

snapshots[
    "test_benefits 1"
] = """

<div class="grid-benefits row col-sm-10 col-lg-8 col-center">
    
    
    
    
        <h3 class="grid-benefits__title grid-benefits__title1">Test title</h3>
    
    
        <h3 class="grid-benefits__title grid-benefits__title2">Test title</h3>
    
    
        <h3 class="grid-benefits__title grid-benefits__title3">Test title</h3>
    
    <p class="grid-benefits__text1">Test text</p>
    <p class="grid-benefits__text2">Test text</p>
    <p class="grid-benefits__text3">Test text</p>
    
        <div class="grid-benefits__button grid-benefits__button1">
            <a href="/" class="button button--regular button--secondary button--secondary--dark">Test button</a>
        </div>
    
    
        <div class="grid-benefits__button grid-benefits__button2">
            <a href="/" class="button button--regular button--secondary button--secondary--dark">Test button</a>
        </div>
    
    
        <div class="grid-benefits__button grid-benefits__button3">
            <a href="/" class="button button--regular button--secondary button--secondary--dark">Test button</a>
        </div>
    
</div>
"""

snapshots[
    "test_game_banner 1"
] = """

<div class="banner--game col-center col-lg-10 col-sm-12 test--class">
    <div>
        <h4 class="banner--game__text banner--game__ages">Test ages</h4>
        <h1 class="banner--game__title">Test title</h1>
        <p class="banner--game__text"><strong>Test description</strong></p>
        <a href="/play/"
           class="button button--big button-primary button--primary--general-educate">Test button</a>
    </div>
</div>
"""

snapshots[
    "test_hero_card 1"
] = """

<div class="background">
    <div class="card col-sm-8 col-center">
        <img class="card__image" src="/static/portal/img/kurono_landing_hero.png">
        <div class="card__text">
            <h3 class="card__title">Test title</h3>
            <p>Test description</p>
            <div class="button-group">
                <a href="/" class="button button--primary--general-play">
                    Test button 1<div class="glyphicon glyphicon-menu-right"></div>
                </a>
                <a href="/" class="button button--primary--general-play">
                    Test button 2<div class="glyphicon glyphicon-menu-right"></div>
                </a>
            </div>
        </div>
    </div>
</div>
"""

snapshots[
    "test_card_list 1"
] = """

<div class="background">
    <div class="grid-worksheets col-sm-8 col-center">
        
            <div class="card">
                <div class="card__images">
                    <img class="card__image" src="/static/portal/img/get_creative.png">
                    
                        <img class="card__thumbnail" src="/static/portal/img/sadface.png">
                    
                </div>
                <div class="card__text">
                    <h3 class="card__title">Test card 1</h3>
                    
                        <p>Test description 1</p>
                    
                </div>
            </div>
        
            <div class="card">
                <div class="card__images">
                    <img class="card__image" src="/static/portal/img/get_creative.png">
                    
                        <h3 class="card__thumbnail">Coming Soon</h3>
                    
                </div>
                <div class="card__text">
                    <h3 class="card__title">Test card 2</h3>
                    
                        <p>Test description 2</p>
                    
                </div>
            </div>
        
            <div class="card">
                <div class="card__images">
                    <img class="card__image" src="/static/portal/img/get_creative.png">
                    
                        <h3 class="card__thumbnail">Coming Soon</h3>
                    
                </div>
                <div class="card__text">
                    <h3 class="card__title">Test card 3</h3>
                    
                        <p>Test description 3</p>
                    
                </div>
            </div>
        
            <div class="card">
                <div class="card__images">
                    <img class="card__image" src="/static/portal/img/get_creative.png">
                    
                        <h3 class="card__thumbnail">Coming Soon</h3>
                    
                </div>
                <div class="card__text">
                    <h3 class="card__title">Test card 4</h3>
                    
                        <p>Test description 4</p>
                    
                </div>
            </div>
        
            <div class="card">
                <div class="card__images">
                    <img class="card__image" src="/static/portal/img/get_creative.png">
                    
                        <h3 class="card__thumbnail">Coming Soon</h3>
                    
                </div>
                <div class="card__text">
                    <h3 class="card__title">Test card 5</h3>
                    
                        <p>Test description 5</p>
                    
                </div>
            </div>
        
            <div class="card">
                <div class="card__images">
                    <img class="card__image" src="/static/portal/img/get_creative.png">
                    
                </div>
                <div class="card__text">
                    <h3 class="card__title">Test card 6</h3>
                    
                        <div class="button-group">
                            <a href="/" class="button button--primary--general-play">
                                Test button
                            </a>
                        </div>
                    
                </div>
            </div>
        
    </div>
</div>
"""

snapshots[
    "test_character_list 1"
] = """


<div class="background">
    <div class="grid-characters col-lg-9 col-md-11 col-sm-8">
        
            <div class="card">
                <h1 class="card__title">Xian</h1>
                <img class="card__image" src="https://storage.googleapis.com/codeforlife-assets/images/Xian.png">
                <p class="card__text">Fun, active, will dance to just about anything that produces a beat. Has great memory, always a joke at hand, might try to introduce memes in Ancient Greece. Scored gold in a track race once and will take any opportunity to bring that up.</p>
            </div>
        
            <div class="card">
                <h1 class="card__title">Jools</h1>
                <img class="card__image" src="https://storage.googleapis.com/codeforlife-assets/images/Jools.png">
                <p class="card__text">A quick-witted kid who wasn’t expecting to embark in a time-warping journey but can’t say no to a challenge. Someone has to keep the rest of the group in check, after all!</p>
            </div>
        
            <div class="card">
                <h1 class="card__title">Zayed</h1>
                <img class="card__image" src="https://storage.googleapis.com/codeforlife-assets/images/Zayed.png">
                <p class="card__text">A pretty chill, curious soul that prefers practice to theory. Always ready to jump into an adventure if it looks interesting enough; not so much otherwise. Probably the one who accidentally turned the time machine on in first place.</p>
            </div>
        
    </div>
</div>
"""



# coding=utf-8
AIMMO_MAIN_HEADLINE = {
    "title": "Join a coding adventure!",
    "description": "Whether you’re a teacher or a student, Kurono makes learning to code easy and great fun. Using Python, you can travel through time to collect all the valuable missing museum artifacts.",
}

AIMMO_CHALLENGES_HEADLINE = {
    "title": "Challenges",
    "description": "Individual and collaborative challenges keep students engaged and having fun. "
    "(Coming soon)",
}

AIMMO_RESOURCES_HEADLINE = {
    "title": "Teaching Resources",
    "description": "Integrate Kurono in your classroom with our printable student challenges, teacher guidance sheets and supporting materials.",
}

AIMMO_PLAY_ONLINE_HEADLINE = {
    "title": "Play Online",
    "description": "No need to install anything, so you can get started right away.",
}

AIMMO_BENEFITS = {
    "first": {
        "image": "portal/img/icon_laptop.png",
        "title": "Aligned to the curriculum",
        "text": "Kurono is aligned to the National Computing Curriculum.",
        "button": {"text": "", "link": ""},
    },
    "second": {
        "image": "portal/img/icon_free.png",
        "title": "Completely free to use",
        "text": "We are also open source!",
        "button": {"text": "", "link": ""},
    },
    "third": {
        "image": "portal/img/icon_multiplayer.svg",
        "title": "Online multiplayer game",
        "text": "Classes play together in the same world.",
        "button": {"text": "", "link": ""},
    },
}



# -*- coding: utf-8 -*-
# Code for Life
#
# Copyright (C) 2019, Ocado Limited
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ADDITIONAL TERMS – Section 7 GNU General Public Licence
#
# This licence does not grant any right, title or interest in any “Ocado” logos,
# trade names or the trademark “Ocado” or any other trademarks or domain names
# owned by Ocado Innovation Limited or the Ocado group of companies or any other
# distinctive brand features of “Ocado” as may be secured from time to time. You
# must not distribute any modification of this program using the trademark
# “Ocado” or claim any affiliation or association with Ocado or its employees.
#
# You are not authorised to use the name Ocado (or any of its trade names) or
# the names of any author or contributor in advertising or for publicity purposes
# pertaining to the distribution of this program, without the prior written
# authorisation of Ocado.
#
# Any propagation, distribution or conveyance of this program must include this
# copyright notice and these terms. You must not misrepresent the origins of this
# program; modified versions of the program must be marked as such and not
# identified as the original program.
from django.core.urlresolvers import reverse
from django.test import Client, TestCase

from common.models import Teacher
from common.tests.utils.classes import create_class_directly
from common.tests.utils.student import create_school_student_directly
from common.tests.utils.teacher import signup_teacher_directly
from deploy import captcha

from .utils.organisation import (
    create_organisation_directly,
    join_teacher_to_organisation,
)


class TestTeacherViews(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.email, cls.password = signup_teacher_directly()
        _, _, cls.class_access_code = create_class_directly(cls.email)
        create_school_student_directly(cls.class_access_code)

    def login(self):
        c = Client()
        assert c.login(username=self.email, password=self.password)
        return c

    def test_reminder_cards(self):
        c = self.login()
        url = reverse("teacher_print_reminder_cards", args=[self.class_access_code])
        response = c.get(url)
        self.assertEqual(response.status_code, 200)

    def test_organisation_kick_has_correct_permissions(self):
        teacher2_email, _ = signup_teacher_directly()
        org_name, org_postcode = create_organisation_directly(self.email)
        join_teacher_to_organisation(self.email, org_name, org_postcode, is_admin=True)
        join_teacher_to_organisation(teacher2_email, org_name, org_postcode)
        teacher2_id = Teacher.objects.get(new_user__email=teacher2_email).id

        client = self.login()
        url = reverse("organisation_kick", args=[teacher2_id])
        response = client.get(url)
        assert response.status_code == 405
        response = client.post(url)
        assert response.status_code == 302


class TestLoginViews(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.orig_captcha_enabled = captcha.CAPTCHA_ENABLED
        captcha.CAPTCHA_ENABLED = False
        super(TestLoginViews, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        captcha.CAPTCHA_ENABLED = cls.orig_captcha_enabled
        super(TestLoginViews, cls).tearDownClass()

    def _set_up_test_data(self):
        teacher_email, teacher_password = signup_teacher_directly()
        create_organisation_directly(teacher_email)
        _, _, class_access_code = create_class_directly(teacher_email)
        student_name, student_password, _ = create_school_student_directly(
            class_access_code
        )

        return (
            teacher_email,
            teacher_password,
            student_name,
            student_password,
            class_access_code,
        )

    def _create_and_login_teacher(self, next_url=False):
        email, password, _, _, _ = self._set_up_test_data()

        if next_url:
            url = reverse("teacher_login") + "?next=/"
        else:
            url = reverse("teacher_login")

        c = Client()
        response = c.post(
            url,
            {
                "username": email,
                "password": password,
                "g-recaptcha-response": "something",
            },
        )
        return response, c

    def _create_and_login_school_student(self, next_url=False):
        _, _, name, password, class_access_code = self._set_up_test_data()

        if next_url:
            url = reverse("student_login") + "?next=/"
        else:
            url = reverse("student_login")

        c = Client()
        response = c.post(
            url,
            {
                "username": name,
                "access_code": class_access_code,
                "password": password,
                "g-recaptcha-response": "something",
            },
        )
        return response, c

    def test_teacher_login_redirect(self):
        response, _ = self._create_and_login_teacher(True)
        self.assertRedirects(response, "/")

    def test_student_login_redirect(self):
        response, _ = self._create_and_login_school_student(True)
        self.assertRedirects(response, "/")

    def test_teacher_already_logged_in_login_page_redirect(self):
        _, c = self._create_and_login_teacher()

        url = reverse("teacher_login")
        response = c.get(url)
        self.assertRedirects(response, "/teach/dashboard/")

    def test_student_already_logged_in_login_page_redirect(self):
        _, c = self._create_and_login_school_student()

        url = reverse("student_login")
        response = c.get(url)
        self.assertRedirects(response, "/play/details/")

    def test_teacher_already_logged_in_register_page_redirect(self):
        _, c = self._create_and_login_teacher()

        url = reverse("register")
        response = c.get(url)
        self.assertRedirects(response, "/teach/dashboard/")

    def test_student_already_logged_in_register_page_redirect(self):
        _, c = self._create_and_login_school_student()

        url = reverse("register")
        response = c.get(url)
        self.assertRedirects(response, "/play/details/")


class TestViews(TestCase):
    def test_covid_response_page(self):
        c = Client()
        home_url = reverse("home")
        response = c.get(home_url)

        bytes = response.__dict__["_container"][0]
        html = bytes.decode("utf8")

        page_url = reverse("home-learning")

        expected_html = f"""    <div id="messages">
        
        <div class="sub-nav--message">
            
            <div class="sub-nav safe message__home-learning success">
                
                <img title="Information" alt="Information sign" src="/static/portal/img/icon_info.png">
                
                <p>
                    Families: #KeepKidsCoding! <a href='{page_url}'>Tap here</a> for free, easy, home coding lessons.</p>
                <a class="x-icon"><img title="Close" alt="Close sign" src="/static/portal/img/icon_close.png"></a>
            </div>
            
        </div>
        
    </div>"""

        self.assertIn(expected_html, html)

        response = c.get(page_url)

        self.assertEquals(200, response.status_code)



from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect, render
from django.contrib.auth.views import LoginView
from django.contrib.auth.forms import AuthenticationForm

# This import is required so that 2FA works properly
from portal import handlers

from portal.forms.teach import TeacherLoginForm
from portal.views.home import redirect_teacher_to_correct_page
from common.permissions import logged_in_as_teacher
from common.utils import using_two_factor


class TeacherLoginView(LoginView):
    template_name = "portal/login/teacher.html"
    form_class = TeacherLoginForm

    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            if logged_in_as_teacher(request.user):
                return redirect(reverse_lazy("dashboard"))
            return redirect(reverse_lazy("home"))
        return super(TeacherLoginView, self).get(request, *args, **kwargs)

    def get_success_url(self):
        url = self.get_redirect_url()
        return url or redirect_teacher_to_correct_page(
            self.request, self.request.user.userprofile.teacher
        )

    def form_valid(self, form):
        user = form.get_user()
        if using_two_factor(user):
            return render(
                self.request,
                "portal/2FA_redirect.html",
                {
                    "form": AuthenticationForm(),
                    "username": user.username,
                    "password": form.cleaned_data["password"],
                },
            )
        return super(TeacherLoginView, self).form_valid(form)
